// file    : CheckBoxReader.cpp
// brief   : defines "control reader" for check boxes
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-03-01 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "CheckBoxReader.h"

static void OnFocusGained_JCheckBox( ISpeakable * speakerRef, AccessibleContextInfo & contextInfo );
static void OnFocusGained_JCheckBoxMenuItem( ISpeakable * speakerRef, AccessibleContextInfo & contextInfo );


//=========================================================
//
//  Init / Dispose
//
//=========================================================

CheckBoxReader::CheckBoxReader( ISpeakable * speaker )
	: ControlReader( speaker )
{}

CheckBoxReader::~CheckBoxReader()
{}


//=========================================================
//
//  ControlReader interface
//
//=========================================================

//-------------------------------------
// function : CheckBoxReader::CanHandleContext
// brief    : determine whether this class can handle the event sent from AccessBridge
//-------------------------------------
bool
CheckBoxReader::CanHandleContext( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	// check if the context was from proper Swing control
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_CHECK_BOX) != 0 )
	{
		return false;
	}

	return true;
	vmID; context;
}

//-------------------------------------
// function : CheckBoxReader::OnFocusGained
//-------------------------------------
void
CheckBoxReader::OnFocusGained( long vmID, FocusEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	// do proper action according to the component type; JCheckBox or JCheckBoxMenuItem
	if( wcsstr(contextInfo.states_en_US, L"focused") != 0 )
	{
		OnFocusGained_JCheckBox( _SpeakerRef, contextInfo );
	}
	else if( getParentWithRole(vmID, context, L"menu") != NULL )
	{
		OnFocusGained_JCheckBoxMenuItem( _SpeakerRef, contextInfo );
	}
	//else: Focus *Lost* event for a JCheckBox or some

	// do parent action
	ControlReader::OnFocusGained( vmID, event, context, contextInfo );
}

static void
OnFocusGained_JCheckBox( ISpeakable * speakerRef, AccessibleContextInfo & contextInfo )
{
	wstring	checkState;

	// get check state
	if( wcsstr(contextInfo.states_en_US, L"checked") != 0 )
	{
		checkState = Utl::GetString( "CheckBoxReader.CheckStateOn", "checked" );
	}
	else
	{
		checkState = Utl::GetString( "CheckBoxReader.CheckStateOff", "unchecked" );
	}

	// generate text to speech
	speakerRef->ShutUp();
	speakerRef->Speak( contextInfo.name );
	speakerRef->Speak( L" " );
	speakerRef->Speak( checkState );
	speakerRef->Wait( true );
}

static void
OnFocusGained_JCheckBoxMenuItem( ISpeakable * speakerRef, AccessibleContextInfo & contextInfo )
{
	// currently, read same as check box control
	OnFocusGained_JCheckBox( speakerRef, contextInfo );
}

//-------------------------------------
// function : CheckBoxReader::OnStatePropertyChange
//-------------------------------------
void
CheckBoxReader::OnStatePropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo, wchar_t * oldState, wchar_t * newState )
{
	bool			prevChecked = false;
	bool			nextChecked = false;
	wstring			checkState;
	
	// if it's an event for other state, ignore this
	prevChecked = (  wcsstr(oldState, L"checked") != 0  );
	nextChecked = (  wcsstr(newState, L"checked") != 0  );
	if( prevChecked == false && nextChecked == false )
	{
		return; // not an event for check state
	}

	// get check state string
	if( nextChecked )
		checkState = Utl::GetString( "CheckBoxReader.CheckStateOn", "checked" );
	else
		checkState = Utl::GetString( "CheckBoxReader.CheckStateOff", "unchecked" );

	// speak
	_SpeakerRef->ShutUp();
	_SpeakerRef->Speak( checkState );
	_SpeakerRef->Wait( false );
	_SpeakerRef->Speak( contextInfo.name );
	_SpeakerRef->Wait( true );

	// do parent action
	ControlReader::OnStatePropertyChange( vmID, event, context, contextInfo, oldState, newState );
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
