// file    : ComboBoxReader.cpp
// brief   : defines "control reader" for combo boxes
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-03-01 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "ComboBoxReader.h"

inline static bool IsKeyDown( unsigned int  keyCode );

//=========================================================
//
//  Init / Dispose
//
//=========================================================

ComboBoxReader::ComboBoxReader( ISpeakable * speaker )
	: ControlReader( speaker )
{
	_TextReader = new TextReader( speaker );
	_ListReader = new ListReader( speaker );
}

ComboBoxReader::~ComboBoxReader()
{
	delete _TextReader;
	delete _ListReader;
}


//=========================================================
//
//  ControlReader interface
//
//=========================================================

//-------------------------------------
// function : ComboBoxReader::CanHandleContext
// brief    : determine whether this class can handle the event sent from AccessBridge
//-------------------------------------
bool
ComboBoxReader::CanHandleContext( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	// if it's ComboBox, okay to handle it of course.
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_COMBO_BOX) == 0 )
	{
		return true;
	}
	// and if it's a list box, it might be one that used inside a combo-box
	else if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_LIST) == 0 )
	{
		AccessibleContext		parent;
		AccessibleContextInfo	parentInfo;

		// get parent's parent's parent's parent.
		parent = context;
		for( int i=0; i<4; i++ )
		{
			parent = GetAccessibleParentFromContext( vmID, parent );
			if( parent == NULL )
				return false; // this must not be in a combo box
		}

		// if that is not a combo box, ignore this event
		GetAccessibleContextInfo( vmID, parent, &parentInfo );
		if( wcsstr(parentInfo.role_en_US, ACCESSIBLE_COMBO_BOX) == NULL )
		{
			return false;
		}

		return true;
	}
	// and if it's a edit box, it might be one that used inside a combo-box
	else if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_TEXT) == 0 )
	{
		AccessibleContext		parent;
		AccessibleContextInfo	parentInfo;

		parent = GetAccessibleParentFromContext( vmID, context );
		GetAccessibleContextInfo( vmID, parent, &parentInfo );
		if( wcsstr(parentInfo.role_en_US, ACCESSIBLE_COMBO_BOX) != NULL )
		{
			return true;
		}
	}

	return false;
}

//-------------------------------------
// function : ComboBoxReader::OnFocusGained
//-------------------------------------
void
ComboBoxReader::OnFocusGained( long vmID, FocusEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	AccessibleContext		comboBox;
	AccessibleContextInfo	comboBoxInfo;
	wstring					label;
	wstring					namePart;
	wstring					content;

	// in some cases, this event was occur when the OTHER control have focused.
	// so in such cases, ignore this event.
	if( wcsstr(contextInfo.states_en_US, L"focused") == NULL )
	{
		return;
	}

	// get ComboBox object
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_COMBO_BOX) == 0 )
	{
		// this is an uneditorble combo box so we can use usual way to get label.
		comboBox = context;
		comboBoxInfo = contextInfo;
	}
	else
	{
		// this is an editable combo box. so can NOT use usual way to get label
		// because the text box got focus is a child of the combo box
		// therefore the label for the combo box is not brother of this text box.
		comboBox = GetAccessibleParentFromContext( vmID, context );
		GetAccessibleContextInfo( vmID, comboBox, &comboBoxInfo );
	}

	// get label
	label = Utl::GetLabelFor( vmID, comboBox, comboBoxInfo.indexInParent );
	namePart = Utl::GetString( "ComboBoxReader.NamePart", "{0} combo box" );
	namePart = Utl::Format( namePart.c_str(), label.c_str() );

	// get content
	if( contextInfo.accessibleText == TRUE )
	{
		// uneditable
		AccessibleTextItemsInfo textItems;
		
		// AccessBridge bug? GetAccessibleTextRange always returns null string.
		// so only use first sentence as content for now
		/*AccessibleTextInfo textInfo;
		wchar_t * contextBuf;
		GetAccessibleTextInfo( vmID, context, &textInfo, 0, 0 );
		contextBuf = new wchar_t[ textInfo.charCount+1 ];
		GetAccessibleTextRange( vmID, context, 0, textInfo.charCount, contextBuf, textInfo.charCount );
		delete contextBuf;*/
		GetAccessibleTextItems( vmID, context, &textItems, 0 );
		content = textItems.sentence;
	}
	else if( contextInfo.accessibleSelection == TRUE )
	{
		// editable
		AccessibleContext		selectedObj;
		AccessibleContextInfo	selectedObjInfo;

		selectedObj = GetAccessibleSelectionFromContext( vmID, context, 0 );
		if( selectedObj != NULL )
		{
			GetAccessibleContextInfo( vmID, selectedObj, &selectedObjInfo );
			content = selectedObjInfo.name;
		}
	}

	// speech
	_SpeakerRef->ShutUp();
	_SpeakerRef->Speak( namePart );
	if( content.empty() == false )
	{
		_SpeakerRef->Wait( false );
		_SpeakerRef->Speak( content );
	}
	_SpeakerRef->Wait( true );

	ControlReader::OnFocusGained( vmID, event, context, contextInfo );
}

//-------------------------------------
// function : ComboBoxReader::OnCaretPropertyChange
//-------------------------------------
void
ComboBoxReader::OnCaretPropertyChange( long vmID, CaretEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo, int oldPosition, int newPosition )
{
	// this event can occur as a result of choosing a choice in the list in combo box. in such case, ignore.
	if( IsKeyDown(VK_UP) || IsKeyDown(VK_DOWN) || IsKeyDown(VK_NEXT) || IsKeyDown(VK_PRIOR) )
	{
		return;
	}
	
	// delegate
	_TextReader->OnCaretPropertyChange( vmID, event, context, contextInfo, oldPosition, newPosition );
}

//-------------------------------------
// function : ComboBoxReader::OnSelectionPropertyChange
//-------------------------------------
void
ComboBoxReader::OnSelectionPropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	_ListReader->OnSelectionPropertyChange( vmID, event, context, contextInfo );
}


//=========================================================


inline static bool
IsKeyDown( unsigned int keyCode )
{
	short state = GetKeyState( keyCode );
	return ( state < 0 );
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
