// file    : ListReader.cpp
// brief   : defines "control reader" for list boxes
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-05-27 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "ListReader.h"
#include <assert.h>

static const char* kNoSelectionStr = "no selection";
static const char* kItemPositionStr = "{0} of {1}";
using Utl::to_s;

static int FindSelectedChild( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo, AccessibleContextInfo & out_childInfo );

//=========================================================
//
//  Init / Dispose
//
//=========================================================

//-------------------------------------
// function : ListReader::ListReader
//-------------------------------------
ListReader::ListReader( ISpeakable * speaker )
	: ControlReader( speaker )
{}

//-------------------------------------
// function : ListReader::ListReader
//-------------------------------------
ListReader::~ListReader()
{}


//=========================================================
//
//  ControlReader interface
//
//=========================================================

//-------------------------------------
// function : ListReader::CanHandleContext
// brief    : determine whether this class can handle the event sent from AccessBridge
//-------------------------------------
bool
ListReader::CanHandleContext( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	AccessibleContext		parent;
	AccessibleContextInfo	parentInfo;

	// check if the context was from proper Swing control
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_LIST) != 0 )
	{
		return false;
	}

	// combo-box also has a list box inside. if that list box got a focus, ignore.

	// get parent's parent's parent's parent.
	parent = context;
	for( int i=0; i<4; i++ )
	{
		parent = GetAccessibleParentFromContext( vmID, parent );
		if( parent == NULL )
			return true; // this must not be in a combo box
	}

	// if that is a combo box, ignore this event
	GetAccessibleContextInfo( vmID, parent, &parentInfo );
	if( wcsstr(parentInfo.role_en_US, ACCESSIBLE_COMBO_BOX) != NULL )
	{
		return false;
	}

	return true;
}


//-------------------------------------
// function : ListReader::OnFocusGained
//-------------------------------------
void
ListReader::OnFocusGained( long vmID, FocusEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	int rc; // result code
	bool selectedChildExists = false;
	wstring label;
	wstring namePart;
	wstring selectionPart;
	AccessibleContextInfo childInfo;

	// in some cases, this event was occur when the OTHER control have focused.
	// so in such cases, ignore this event.
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		return;
	}

	// get label
	if( 0 < wcslen(contextInfo.name) )
	{
		label = contextInfo.name;
	}
	else
	{
		label = Utl::GetLabelFor( vmID, context, contextInfo.indexInParent );
	}

	// construct UI element name part of the text
	if( label.empty() == false )
	{
		wstring format = Utl::GetString( "ListReader.NamePart", "{0} list box" );
		namePart = Utl::Format( format.c_str(), label.c_str() );
	}

	// find selected child and get its name
	rc = FindSelectedChild( vmID, context, contextInfo, childInfo );
	if( rc == 0 )
	{
		selectedChildExists = true;
	}

	// then, say selected child's name with JList's label
	_SpeakerRef->ShutUp();
	if( selectedChildExists )
	{
		wstring format = Utl::GetString( "ListReader.ItemPositionPart", kItemPositionStr );
		wstring positionPart = Utl::Format( format.c_str(), to_s(childInfo.indexInParent+1).c_str(), to_s(contextInfo.childrenCount).c_str() );
		_SpeakerRef->Speak( namePart );
		_SpeakerRef->Wait( false );
		_SpeakerRef->Speak( childInfo.name );
		_SpeakerRef->Wait( false );
		_SpeakerRef->Speak( positionPart );
	}
	else
	{
		_SpeakerRef->Speak( namePart );
		_SpeakerRef->Wait( false );
		_SpeakerRef->Speak( Utl::GetString("ListReader.NoSelection", kNoSelectionStr) );
	}
	_SpeakerRef->Wait( true );

	ControlReader::OnFocusGained( vmID, event, context, contextInfo );
}

static int
FindSelectedChild( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo, AccessibleContextInfo & out_childInfo )
{
	AccessibleContext child;

	for( int i=0; i<contextInfo.childrenCount; i++ )
	{
		child = GetAccessibleChildFromContext( vmID, context, i );
		if( child == NULL )
		{
			break;
		}

		GetAccessibleContextInfo( vmID, child, &out_childInfo );
		if( wcsstr(out_childInfo.states_en_US, L"selected") != NULL )
		{
			return 0; // found selected child
		}
	}

	// no selected childs found
	return -1;
}



//virtual void OnSelectionPropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo );
//-------------------------------------
// function : ListReader::OnSelectionPropertyChange
//-------------------------------------
void
//ListReader::OnActiveDescendentPropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo, jobject oldActiveDescendent, jobject newActiveDescendent )
ListReader::OnSelectionPropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	AccessibleContext		selectionContext;
	AccessibleContextInfo	childInfo;
	int						selectionCount;
	wstring					positionPart;

	// ignore event for unfocused component
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		// ...unless it's an exceptional case:
		// a list box inside a combo box has no focus but must handle this event
		AccessibleContext		parent;
		AccessibleContextInfo	parentInfo;
		
		// retrieve the parent of parent of parent of parent
		parent = context;
		for( int i=0; i<4; i++ )
		{
			parent = GetAccessibleParentFromContext( vmID, parent );
			if( parent == NULL )
			{
				return;
			}
		}

		// if its not a combo box, this list is not a inner list of combo
		GetAccessibleContextInfo( vmID, parent, &parentInfo );
		if( wcscmp(parentInfo.role_en_US, ACCESSIBLE_COMBO_BOX) != 0 )
		{
			return;
		}
	}

	// AccessibleSelection available?
	if( contextInfo.accessibleSelection != TRUE )
	{
		// (this case should not be happen but in some cases, happens)
		return;
	}

	// get AccessibleSelection
	selectionCount = GetAccessibleSelectionCountFromContext( vmID, context );
	if( selectionCount <= 0 )
	{
		// selection cleared (?)
		_SpeakerRef->ShutUp();
		_SpeakerRef->Speak( Utl::GetString("ListReader.NoSelection", kNoSelectionStr) );
		return;
	}
	
	// get name of first selected child
	selectionContext = GetAccessibleSelectionFromContext( vmID, context, 0 );
	GetAccessibleContextInfo( vmID, selectionContext, &childInfo );

	// get position of that child
	positionPart = Utl::GetString( "ListReader.ItemPositionPart", kItemPositionStr );
	positionPart = Utl::Format( positionPart.c_str(), to_s(childInfo.indexInParent+1).c_str(), to_s(contextInfo.childrenCount).c_str() );
	
	// speak
	_SpeakerRef->ShutUp();
	_SpeakerRef->Speak( childInfo.name );
	_SpeakerRef->Wait( false );
	_SpeakerRef->Speak( positionPart );

	//ControlReader::OnActiveDescendentPropertyChange( vmID, event, context, contextInfo, oldActiveDescendent, newActiveDescendent );
	ControlReader::OnSelectionPropertyChange( vmID, event, context, contextInfo );
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
