// file    : RadioButtonReader.cpp
// brief   : defines "control reader" for radio buttons
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-05-27 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "RadioButtonReader.h"

static const char * kSelectedStr	= "selected";
static const char * kNotSelectedStr	= "not selected";
static const char * kOnSelectedStr	= "{0} selected";
static const char * kTypeStr		= "radio button";

static void OnFocusGained_JRadioButton( ISpeakable* speaker, long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo );
static void OnFocusGained_JRadioButtonMenuItem( ISpeakable* speaker, AccessibleContextInfo & contextInfo );

static wstring GetLabelForRadioButton( long vmID, AccessibleContext context );


//=========================================================
//
//  Init / Dispose
//
//=========================================================

RadioButtonReader::RadioButtonReader( ISpeakable * speaker )
	: ControlReader( speaker )
{}

RadioButtonReader::~RadioButtonReader()
{}



//=========================================================
//
//  ControlReader interface
//
//=========================================================

//-------------------------------------
// function : RadioButtonReader::CanHandleContext
// brief    : determine whether this class can handle the event sent from AccessBridge
//-------------------------------------
bool
RadioButtonReader::CanHandleContext( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	// check if the context was from proper Swing control
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_RADIO_BUTTON) != 0 )
	{
		return false;
	}

	return true;
	vmID, context;
}

//-------------------------------------
// function : RadioButtonReader::OnFocusGained
//-------------------------------------
void
RadioButtonReader::OnFocusGained( long vmID, FocusEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	// do proper action according to the component type; JCheckBox or JCheckBoxMenuItem
	if( wcsstr(contextInfo.states_en_US, L"focused") != 0 )
	{
		OnFocusGained_JRadioButton( _SpeakerRef, vmID, context, contextInfo );
	}
	else if( getParentWithRole(vmID, context, L"menu") != NULL )
	{
		OnFocusGained_JRadioButtonMenuItem( _SpeakerRef, contextInfo );
	}

	// do parent action
	ControlReader::OnFocusGained( vmID, event, context, contextInfo );
}

static void
OnFocusGained_JRadioButton( ISpeakable* speaker, long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	bool checked = false;
	wstring label;
	wstringstream text;

	// in some cases, this event was occur when the OTHER control have focused.
	// so in such cases, ignore this event.
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		return;
	}

	// get check state
	if( wcsstr(contextInfo.states_en_US, L"checked") != 0 )
	{
		checked = true;
	}

	// get label for this button
	label = GetLabelForRadioButton( vmID, context );
	
	//---- speak ----
	speaker->ShutUp();
	speaker->Speak( contextInfo.name );
	if( checked )
	{
		speaker->Speak( L" " );
		speaker->Speak( Utl::GetString("RadioButtonReader.Selected", kSelectedStr) );
	}
	speaker->Wait( false );
	if( label.empty() )
	{
		speaker->Speak( Utl::GetString("RadioButtonReader.Type", kTypeStr) );
	}
	else
	{
		wstring format = Utl::GetString( "RadioButtonReader.TypeWithLabel", "{0} radio button" );
		speaker->Speak( Utl::Format(format.c_str(), label.c_str()) );
	}
	speaker->Wait( true );
}

static void
OnFocusGained_JRadioButtonMenuItem( ISpeakable* speaker, AccessibleContextInfo & contextInfo )
{
	bool checked = false;

	// when a JRadioButtonMenuItem was selected, that has no focus
	// so we do not check about focus here

	// get check state
	if( wcsstr(contextInfo.states_en_US, L"checked") != 0 )
	{
		checked = true;
	}

	// speak
	speaker->ShutUp();
	speaker->Speak( contextInfo.name );
	if( checked )
	{
		speaker->Speak( L" " );
		speaker->Speak( Utl::GetString("RadioButtonReader.Selected", kSelectedStr) );
	}
	else
	{
		speaker->Speak( L" " );
		speaker->Speak( Utl::GetString("RadioButtonReader.NotSelected", kNotSelectedStr) );
	}
	speaker->Wait( true );
}

//-------------------------------------
// function : RadioButtonReader::OnStatePropertyChange
//-------------------------------------
void
RadioButtonReader::OnStatePropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo, wchar_t * oldState, wchar_t * newState )
{
	bool	nextChecked = false;

	// only do action in case of the radio button was checked
	nextChecked = (  wcsstr(newState, L"checked") != 0  );
	
	if( nextChecked == false )
	{
		return; // not an event for check state
	}

	wstring	format = Utl::GetString( "RadioButtonReader.OnSelected", kOnSelectedStr );

	// speech
	_SpeakerRef->ShutUp();
	if( nextChecked )
	{
		_SpeakerRef->Speak( Utl::Format(format.c_str(), contextInfo.name) );
	}
	_SpeakerRef->Wait( true );

	// do parent action
	ControlReader::OnStatePropertyChange( vmID, event, context, contextInfo, oldState, newState );
}



//=========================================================



//-------------------------------------
// function : GetLabelForRadioButton
// brief    : get label for radio button
// returns  : (success) ... label
//            (failure) ... empty string
// note     : in most cases, this function succeeds but returns empty string
//            because the panel containing the button has no name.
//-------------------------------------
static wstring
GetLabelForRadioButton( long vmID, AccessibleContext context )
{
	int						rc; // result code
	AccessibleContext		parent;
	AccessibleContextInfo	parentInfo;
	
	// get parent
	parent = GetAccessibleParentFromContext( vmID, context );
	if( parent == 0 )
	{
		return wstring();
	}

	// get information about the parent
	rc = GetAccessibleContextInfo( vmID, parent, &parentInfo );
	if( rc == FALSE )
	{
		return wstring();
	}

	// ensure it's a panel
	if( wcscmp(parentInfo.role_en_US, ACCESSIBLE_PANEL) != 0 )
	{
		return wstring();
	}

	// finally, get it's text
	return wstring( parentInfo.name );
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
