// file    : TextReader.cpp
// brief   : defines "control reader" for text boxes
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-11-26 (SGRY)
// license : MIT License (see END of this file)
// note    : some dependence on Win32API was left (use of VK_XXX without wrapping)
//=========================================================
#include "TextReader.h"

inline static bool IsKeyDown( unsigned int keyCode );

//=========================================================
//
//   Init / Dispose
//
//=========================================================

//-------------------------------------
// function : TextReader::TextReader
//-------------------------------------
TextReader::TextReader( ISpeakable * speaker )
	: ControlReader( speaker )
{}

//-------------------------------------
// function : TextReader::TextReader
//-------------------------------------
TextReader::~TextReader()
{}


//=========================================================
//
//  ControlReader interface
//
//=========================================================

//-------------------------------------
// function : TextReader::CanHandleContext
// brief    : determine whether this class can handle the event sent from AccessBridge
//-------------------------------------
bool
TextReader::CanHandleContext( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	AccessibleContext		parent;
	AccessibleContextInfo	parentInfo;

	// check if the context was from proper Swing control
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_TEXT) != 0 )
	{
		return false;
	}

	// combo-box also has a text box inside. if that text box got a focus, ignore.
	parent = GetAccessibleParentFromContext( vmID, context );
	GetAccessibleContextInfo( vmID, parent, &parentInfo );
	if( wcsstr(parentInfo.role_en_US, ACCESSIBLE_COMBO_BOX) != NULL )
	{
		return false;
	}

	return true;
}

//-------------------------------------
// function : TextReader::OnFocusGained
//-------------------------------------
void
TextReader::OnFocusGained( long vmID, FocusEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	AccessibleTextInfo		textInfo;
	AccessibleTextItemsInfo textItems;
	wstring					label;
	wstring					speechText;
	bool					isMultiLine = false;
	wstring					namePart;
	wstring					type;

	// in some cases, this event was occur when the OTHER control have focused.
	// so in such cases, ignore this event.
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		return;
	}

	// get char, word and sentence at where the caret is
	GetAccessibleTextInfo( vmID, context, &textInfo, 0, 0 );
	GetAccessibleTextItems( vmID, context, &textItems, textInfo.caretIndex );

	// get label of this tree control
	if( 0 < wcslen(contextInfo.name) )
	{
		label = contextInfo.name;
	}
	else
	{
		label = Utl::GetLabelFor( vmID, context, contextInfo.indexInParent );
	}

	// get label and text box type (single line or multi-line)
	if( wcsstr(contextInfo.states_en_US, L"multiple line") != NULL )
	{
		isMultiLine = true;
	}

	// make speech text
	namePart = Utl::GetString( "TextReader.NamePart", "{0} {1}" );
	if( isMultiLine )
		type = Utl::GetString( "TextReader.NameMulti", "multi line edit" );
	else
		type = Utl::GetString( "TextReader.NameSingle", "edit" );
	speechText = Utl::Format( namePart.c_str(), label.c_str(), type.c_str() );

	// speech
	_SpeakerRef->ShutUp();
	_SpeakerRef->Speak( speechText );
	_SpeakerRef->Wait( false );
	_SpeakerRef->Speak( textItems.sentence );

	ControlReader::OnFocusGained( vmID, event, context, contextInfo );
}


//-------------------------------------
// function : TextReader::OnCaretPropertyChange
//-------------------------------------
void
TextReader::OnCaretPropertyChange( long vmID, CaretEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo, int oldPosition, int newPosition )
{
	const int READ_CHAR=0, READ_WORD=1, READ_LINE=2, READ_HOME=3, READ_END=4;
	AccessibleTextItemsInfo textItems;
	wstringstream			speechText;
	int						positionDiff;
	int						readType = -1;

	// get key state to determine what to read
	if( IsKeyDown(VK_RIGHT) || IsKeyDown(VK_LEFT) )
	{
		if( IsKeyDown(VK_CONTROL) )
			readType = READ_WORD;
		else
			readType = READ_CHAR;
	}
	else if( IsKeyDown(VK_UP) || IsKeyDown(VK_DOWN)
		|| IsKeyDown(VK_PRIOR) || IsKeyDown(VK_NEXT) )
	{
		readType = READ_LINE;
	}
	else if( IsKeyDown(VK_HOME) )
	{
		readType = READ_HOME;
	}
	else if( IsKeyDown(VK_END) )
	{
		readType = READ_END;
	}

	// if caret property changed not by key, ignore
	if( readType == -1 )
	{
		return;
	}

	// get text items
	positionDiff = newPosition - oldPosition;
	GetAccessibleTextItems( vmID, context, &textItems, newPosition );
	
	// construct speech text
	switch( readType )
	{
		case READ_WORD:
			speechText << textItems.word;
			break;
		case READ_LINE:
			speechText << textItems.sentence;
			break;
		default:
			speechText << textItems.letter;
			break;
	}

	// speak
	_SpeakerRef->ShutUp();
	if( readType == READ_HOME )
	{
		_SpeakerRef->Speak( Utl::GetString("TextReader.LineHead", "line head") );
		_SpeakerRef->Wait( false );
	}
	else if( readType == READ_END )
	{
		_SpeakerRef->Speak( Utl::GetString("TextReader.LineEnd", "line end") );
		_SpeakerRef->Wait( false );
	}
	_SpeakerRef->Speak( speechText.str() );

	ControlReader::OnCaretPropertyChange( vmID, event, context, contextInfo, oldPosition, newPosition );
}



//=============================================================================



inline static bool
IsKeyDown( unsigned int keyCode )
{
	short state = GetKeyState( keyCode );
	return ( state < 0 );
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
