// file    : ToggleButtonReader.cpp
// brief   : defines "control reader" for toggle buttons
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-03-01 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "ToggleButtonReader.h"

static const char* kCheckedStr = "checked";
static const char* kUnCheckedStr = "not checked";

//=========================================================
//
//   Init / Dispose
//
//=========================================================

//-------------------------------------
// function : ToggleButtonReader::ToggleButtonReader
//-------------------------------------
ToggleButtonReader::ToggleButtonReader( ISpeakable * speaker )
	: ControlReader( speaker )
{}

//-------------------------------------
// function : ToggleButtonReader::~ToggleButtonReader
//-------------------------------------
ToggleButtonReader::~ToggleButtonReader()
{}


//=========================================================
//
//    Control Reader interface
//
//=========================================================

//-------------------------------------
// function : ToggleButtonReader::CanHandleContext
// brief    : determine whether this class can handle the event sent from AccessBridge
//-------------------------------------
bool
ToggleButtonReader::CanHandleContext( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	// check if the context was from proper Swing control
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_TOGGLE_BUTTON) != 0 )
	{
		return false;
	}

	return true;
	vmID; context;
}

//-------------------------------------
// function : ToggleButtonReader::OnFocusGained
//-------------------------------------
void
ToggleButtonReader::OnFocusGained( long vmID, FocusEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	wstring checkState;
	wstring	type;

	// in some cases, this event was occur when the OTHER control have focused.
	// so in such cases, ignore this event.
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		return;
	}

	// get the select state of the toggle button
	if( wcsstr(contextInfo.states_en_US, L"checked") != NULL )
		checkState = Utl::GetString( "ToggleButton.Checked", kCheckedStr );
	else
		checkState = Utl::GetString( "ToggleButton.NotChecked", kUnCheckedStr );

	// get localized type name
	type = Utl::GetString( "ToggleButton.Type", "toggle button" );

	// speech
	_SpeakerRef->ShutUp();
	_SpeakerRef->Speak( Utl::Format(L"{0} {1}", contextInfo.name, checkState.c_str()) );
	_SpeakerRef->Wait( false );
	_SpeakerRef->Speak( type );

	ControlReader::OnFocusGained( vmID, event, context, contextInfo );
}


//-------------------------------------
// function : ToggleButtonReader::OnStatePropertyChange
//-------------------------------------
void
ToggleButtonReader::OnStatePropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo, wchar_t * oldState, wchar_t * newState )
{
	bool	nextChecked;
	wstring	checkState;

	// ensure the state change will about selection
	nextChecked = (  wcscmp(newState, L"selected") == 0  );
	if( nextChecked == false && wcscmp(oldState, L"selected") != 0)
	{
		return; // ignore ths event
	}

	// make speech text
	if( nextChecked )
		checkState = Utl::GetString( "ToggleButton.Checked", kCheckedStr );
	else
		checkState = Utl::GetString( "ToggleButton.NotChecked", kUnCheckedStr );

	// speech according the selection state
	_SpeakerRef->ShutUp();
	_SpeakerRef->Speak( checkState );
	_SpeakerRef->Wait( false );
	_SpeakerRef->Speak( contextInfo.name );
	_SpeakerRef->Wait( true );

	// do parent action
	ControlReader::OnStatePropertyChange( vmID, event, context, contextInfo, oldState, newState );
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
