// file    : TreeReader.cpp
// brief   : defines "control reader" for tree components
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-05-27 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "TreeReader.h"

using Utl::to_s;

static int GetBrotherCount( long vmID, AccessibleContext context );
static int GetFirstSelectedChild( long vmID, AccessibleContext node, int nodeDepth, AccessibleContext & outChild );
inline static bool IsTreeItemLabel( AccessibleContextInfo & contextInfo );
inline static int GetNodeDepth( long vmID, AccessibleContext & context );

static int gPrevSelectedDepth = -1;

//=========================================================
//
//  Init / Dispose
//
//=========================================================

//-------------------------------------
// function : TreeReader::TreeReader
//-------------------------------------
TreeReader::TreeReader( ISpeakable * speaker )
	: ControlReader( speaker )
{}

//-------------------------------------
// function : TreeReader::TreeReader
//-------------------------------------
TreeReader::~TreeReader()
{}


//=========================================================
//
//  ControlReader interface
//
//=========================================================

//-------------------------------------
// function : TreeReader::CanHandleContext
// brief    : determine whether this class can handle the event sent from AccessBridge
//-------------------------------------
bool
TreeReader::CanHandleContext( long vmID, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	// check if the context was from proper Swing control
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_TREE) != 0
		&& IsTreeItemLabel(contextInfo) == false )
	{
		return false;
	}

	return true;
	vmID; context;
}


//-------------------------------------
// function : TreeReader::OnFocusGained
//-------------------------------------
void
TreeReader::OnFocusGained( long vmID, FocusEvent event, AccessibleContext context, AccessibleContextInfo & contextInfo )
{
	BOOL					rc; // result code
	AccessibleContext		firstSelectedChild;
	int						firstSelectedChildDepth = -1;
	AccessibleContextInfo	firstSelectedChildInfo;
	wstring					label;
	bool					expanded = false, collapsed = false;
	wstring					namePart;

	// in some cases, this event was occur when the OTHER control have focused.
	// so in such cases, ignore this event.
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		return;
	}

	//---- get selected child's name ----
	// get name of the selected (first) child
	if( contextInfo.accessibleSelection == FALSE )
	{
		goto get_label;
	}
	firstSelectedChildDepth = GetFirstSelectedChild( vmID, context, -1, firstSelectedChild );
	if( firstSelectedChildDepth == -1 )
	{
		goto get_label; // not found
	}

	// get first selected child's information
	rc = GetAccessibleContextInfo( vmID, firstSelectedChild, &firstSelectedChildInfo );
	if( rc == FALSE )
	{
		firstSelectedChildDepth = -1; // this variable also used as a flag later
		goto get_label;
	}
	if( wcsstr(firstSelectedChildInfo.states_en_US, L"expanded") != 0 )
		expanded = true;
	if( wcsstr(firstSelectedChildInfo.states_en_US, L"collapsed") != 0 )
		collapsed = true;

get_label:
	// get label of this tree control
	if( 0 < wcslen(contextInfo.name) )
	{
		label = contextInfo.name;
	}
	else
	{
		label = Utl::GetLabelFor( vmID, context, contextInfo.indexInParent );
	}

	// construct UI element name part of the text
	if( label.empty() == false )
	{
		wstring format = Utl::GetString( "TreeReader.NamePart", "{0} tree" );
		namePart = Utl::Format( format.c_str(), label.c_str() );
	}

	// say label
	_SpeakerRef->ShutUp();
	_SpeakerRef->Speak( namePart );
	_SpeakerRef->Wait( false );

	// say selected node name with open/close state if it has childs
	if( firstSelectedChildDepth != -1 )
	{
		wstring childCountPart;

		// say node name
		_SpeakerRef->Speak( firstSelectedChildInfo.name );
		_SpeakerRef->Speak( L" " );

		// say node open/close state
		if( expanded )
			_SpeakerRef->Speak( Utl::GetString("TreeReader.Open", "open") );
		else if( collapsed )
			_SpeakerRef->Speak( Utl::GetString("TreeReader.Close", "close") );
		_SpeakerRef->Wait( false );
		
		// say number of children
		childCountPart = Utl::GetString( "TreeReader.ChildCountPart", "{0} items" );
		childCountPart = Utl::Format( childCountPart.c_str(), to_s(firstSelectedChildInfo.childrenCount).c_str() );
		_SpeakerRef->Speak( childCountPart );
	}
	// or, say that no nodes are currently selected
	else
	{
		_SpeakerRef->Speak( Utl::GetString("TreeReader.NoSelection", "no selection") );
	}

	ControlReader::OnFocusGained( vmID, event, context, contextInfo );
}


//-------------------------------------
// function : TreeReader::OnStatePropertyChange
// notes    : param "context" indicates not tree but the node which state was changed.
//-------------------------------------
void
TreeReader::OnStatePropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo, wchar_t * oldState, wchar_t * newState )
{
	bool			prevExpanded = false;
	bool			nextExpanded = false;

	// if it's not an expand/collapse state, ignore this
	prevExpanded = (  wcsstr(oldState, L"expanded") != 0  );
	nextExpanded = (  wcsstr(newState, L"expanded") != 0  );
	if( prevExpanded == false && nextExpanded == false )
	{
		return; // not an event for check state
	}

	// speak
	_SpeakerRef->ShutUp();
	if( nextExpanded )
	{
		wstring childCountPart = Utl::GetString( "TreeReader.ChildCountPart", "{0} items" );

		// open/close state
		_SpeakerRef->Speak( Utl::GetString("TreeReader.Open", "open") );
		_SpeakerRef->Wait( false );

		// node name
		_SpeakerRef->Speak( contextInfo.name );
		_SpeakerRef->Wait( false );

		// number of children
		_SpeakerRef->Speak(
				Utl::Format( childCountPart.c_str(), to_s(contextInfo.childrenCount).c_str() )
			);
	}
	else
	{
		wstring nodePositionPart = Utl::GetString( "TreeReader.NodePositionPart", "{0} of {1}" );

		// get parent child count
		int brotherCount = GetBrotherCount( vmID, context );
		if( brotherCount == -1 )
		{
			std::cerr << "failed to get count of brothers / TreeReader::OnStatePropertyChange." << std::endl;
			abort(); return;
		}

		// open/close state
		_SpeakerRef->Speak( Utl::GetString("TreeReader.Close", "close") );
		_SpeakerRef->Wait( false );

		// node name
		_SpeakerRef->Speak( contextInfo.name );
		_SpeakerRef->Wait( false );

		// number of children
		_SpeakerRef->Speak(
				Utl::Format(nodePositionPart.c_str(), to_s(brotherCount).c_str(), to_s(contextInfo.indexInParent+1).c_str() )
			);
	}

	ControlReader::OnStatePropertyChange( vmID, event, context, contextInfo, oldState, newState );
}

//-------------------------------------
// function : TreeReader::OnActiveDescendentPropertyChange
//-------------------------------------
void
TreeReader::OnActiveDescendentPropertyChange( long vmID, jobject event, AccessibleContext context, AccessibleContextInfo & contextInfo, jobject oldActiveDescendent, jobject newActiveDescendent )
{
	BOOL					rc; // result code
	bool					expanded	= false;
	bool					expandable	= false;
	int						depth		= -1;
	AccessibleContextInfo	activeDescendentInfo;
	bool					sayDepth	= false;

	rc = GetAccessibleContextInfo( vmID, newActiveDescendent, &activeDescendentInfo );
	if( rc == FALSE )
	{
		goto parent_action;
	}

	// get depth of the node
	depth = GetNodeDepth( vmID, newActiveDescendent );
	if( depth < 0 )
	{
		goto parent_action; // invalid
	}
	if( gPrevSelectedDepth != depth )
	{
		sayDepth = true;
	}
	gPrevSelectedDepth = depth;

	// get node state
	expandable = ( wcsstr(activeDescendentInfo.states_en_US, L"expandable") != NULL );
	if( expandable )
	{
		expanded = ( wcsstr(activeDescendentInfo.states_en_US, L"expanded") != NULL );
	}

	//--- speak ---
	_SpeakerRef->ShutUp();

	// say depth
	if( sayDepth )
	{
		wstring depthPart;
		depthPart = Utl::GetString( "TreeReader.DepthPart", "depth {0}" );
		depthPart = Utl::Format( depthPart.c_str(), to_s(depth).c_str() );
		_SpeakerRef->Speak( depthPart );
		_SpeakerRef->Wait( false );
	}

	// say node name
	_SpeakerRef->Speak( activeDescendentInfo.name );
	
	// say open/close state
	if( expandable )
	{
		_SpeakerRef->Speak( L" " );
		if( expanded )
			_SpeakerRef->Speak( Utl::GetString("TreeReader.Open", "open") );
		else
			_SpeakerRef->Speak( Utl::GetString("TreeReader.Close", "close") );
	}
	_SpeakerRef->Wait( true );

parent_action:
	// do parent action
	ControlReader::OnActiveDescendentPropertyChange( vmID, event, context, contextInfo, oldActiveDescendent, newActiveDescendent );
}


//=============================================================================


static int
GetBrotherCount( long vmID, AccessibleContext context )
{
	BOOL					rc; // result code
	AccessibleContext		parent;
	AccessibleContextInfo	parentInfo;

	// get parent
	parent = GetAccessibleParentFromContext( vmID, context );
	if( parent == 0 )
	{
		return -1;
	}

	// get parent child count
	rc = GetAccessibleContextInfo( vmID, parent, &parentInfo );
	if( rc == FALSE )
	{
		return -1;
	}

	return parentInfo.childrenCount;
}

// args    : nodeDepth ... caller must set this to -1 (var for recursive processing)
// returns : depth of the child (to be returned to caller)
static int
GetFirstSelectedChild( long vmID, AccessibleContext node, int nodeDepth, AccessibleContext & outChild )
{
	int						rc; // result code
	AccessibleContext		child;
	AccessibleContextInfo	info;

	// if this node has focus, return it
	rc = ::GetAccessibleContextInfo( vmID, node, &info );
	if( 0 <= nodeDepth // "depth==-1" means that the node is tree itself
		&& wcsstr(info.states_en_US, L"selected") != NULL
		&& rc == TRUE )
	{
		outChild = node;
		return nodeDepth;
	}

	// recursively search for childrens
	for( int i=0; i<info.childrenCount; i++ )
	{
		// get [i]th child
		child = ::GetAccessibleChildFromContext( vmID, node, i );

		// call this function recursively
		rc = GetFirstSelectedChild( vmID, child, nodeDepth+1, outChild );
		if( 0 <= rc )
		{
			return rc; // already found. needed no more.
		}
	}

	// return -1 until the focused node was found
	return -1;
}

inline static bool
IsTreeItemLabel( AccessibleContextInfo & contextInfo )
{
	// a label that expandable should be a JTree's item label
	if( wcscmp(contextInfo.role_en_US, ACCESSIBLE_LABEL) == 0
		&& wcsstr(contextInfo.states_en_US, L"expandable") != NULL )
	{
		return true;
	}

	return false;
}

inline static int
GetNodeDepth( long vmID, AccessibleContext & context )
{
	int						rc; // result code
	int						depth = 0;
	AccessibleContext		parent;
	AccessibleContextInfo	parentInfo;
	
	// get direct parent
	parent = GetAccessibleParentFromContext( vmID, context );
	while( parent != NULL )
	{
		// check if the parent is the container tree
		rc = GetAccessibleContextInfo( vmID, parent, &parentInfo );
		if( rc == FALSE )
		{
			break; // error
		}

		if( wcscmp(parentInfo.role_en_US, ACCESSIBLE_TREE) == 0 )
		{
			// found container tree
			return depth;
		}

		// get parent
		parent = ::GetAccessibleParentFromContext( vmID, parent );
		depth++;
	}

	return -1;
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
