/* file		Ini.cpp
 * brief	".ini" file manipulation class
 * date		2007-06-15 (SGRY)
 * license	MIT License (see end of this file)
 **********************************************/
#include "Ini.h"

#include <string>
#include <vector>
#include <algorithm>
#include <functional>
#include <cassert>

#ifdef _WIN32
#	include <windows.h>
#endif


using std::endl;
using std::fstream;
using std::vector;
using Sgry::Ini;
using Sgry::string;
using Sgry::wstring;

using std::find;
using std::getline;


typedef Ini::Entry			Entry;
typedef Ini::Section		Section;
typedef vector<Entry>		EntryVec;
typedef vector<Section>		SectionVec;


//=============================================================================
//
//      Prototype of Inner Function
//
//=============================================================================
static short ParseSectionLine( const wstring & line, wstring & outName );
static short ParseEntryLine( const wstring & line, wstring & outName, wstring & outValue );
static wstring Trim( wstring & str );

static EntryVec::iterator			FindEntryByName( EntryVec & entries, const wstring & name );
static EntryVec::const_iterator		FindEntryByName( const EntryVec & entries, const wstring & name );
static SectionVec::iterator			FindSectionByName( SectionVec & entries, const wstring & name );
static SectionVec::const_iterator	FindSectionByName( const SectionVec & entries, const wstring & name );

inline bool	IsWhiteSpaceChar( wchar_t ch ){ return (ch == L' ' || ch == L'\t'); }


//=============================================================================
//
//      public methods
//
//=============================================================================

//-------------------------------------
//   function	Ini::Ini
//   brief		initialize
//-------------------------------------
Ini::Ini()
{
}

//-------------------------------------
//   function	Ini::set
//! @brief		Set value of specified entry.
//! @retval		(success) ... value
//! @retval		(failure) ... empty string
//-------------------------------------
short
Ini::set( const wstring & section, const wstring & entry, const wstring & value )
{
	EntryVec::iterator		entPos;
	SectionVec::iterator	secPos;
	EntryVec *				entries;

	// get or create specified section from all sections
	secPos = FindSectionByName( m_Sections, section );
	if( secPos != m_Sections.end() )
	{
		// found
		entries = &(secPos->entries);
	}
	else
	{
		// not found
		Section	newSection;
		
		// create new node
		newSection.name = section;
		m_Sections.push_back( newSection );
		
		entries = &( m_Sections[m_Sections.size()-1].entries );
	}
	
	// find specified entry in the section
	entPos = FindEntryByName( *entries, entry );
	if( entPos != entries->end() )
	{
		// found
		// overwrite the entry's value
		entPos->value = value;
	}
	else
	{
		// not found
		Entry newEntry;

		// create the entry
		newEntry.name = entry;
		newEntry.value = value;
		entries->push_back( newEntry );
	}

	return 0;
}


//-------------------------------------
//   function	Ini::set
//! @brief		Set value of specified entry.
//! @retval		(success) ... value of the entry
//! @retval		(failure) ... empty string
//-------------------------------------
short
Ini::set( const string & section, const string & entry, const wstring & value )
{
	return set( widen(section), widen(entry), value );
}


//-------------------------------------
//   function	Ini::set
//! @brief		Set value of specified entry.
//! @retval		(success) ... value of the entry
//! @retval		(failure) ... empty string
//-------------------------------------
short
Ini::set( const wstring & section, const wstring & entry, const string & value, CodePage cp )
{
	return set( section, entry, widen(value,cp) );
}


//-------------------------------------
//   function	Ini::get
//! @brief		Get value of specified entry.
//! @retval		(success) ... value of the entry
//! @retval		(failure) ... empty string
//-------------------------------------
wstring
Ini::get( const wstring & section, const wstring & entry ) const
{
	EntryVec::const_iterator	entPos;
	SectionVec::const_iterator	secPos;
	const EntryVec *			entries;

	// find specified section in all sections
	secPos = FindSectionByName( m_Sections, section );
	if( secPos == m_Sections.end() )
	{
		// not found
		return wstring();
	}
	entries = &(secPos->entries);

	// find specified entry in the section
	for( entPos=entries->begin(); entPos!=entries->end(); entPos++ )
	{
		if( entPos->name == entry )
		{
			// overwrite the entry's value
			return entPos->value;
		}
	}
	
	// not found
	return wstring();
}


//-------------------------------------
//   function	Ini::get
//! @brief		Get value of specified entry.
//! @retval		(success) ... value of the entry
//! @retval		(failure) ... empty string
//-------------------------------------
wstring
Ini::get( const string & section, const string & entry ) const
{
	return get( widen(section), widen(entry) );
}


//-------------------------------------
//   function	Ini::get
//! @brief		Get value of specified entry.
//! @retval		(success) ... value of the entry
//! @retval		(failure) ... empty string
//-------------------------------------
std::string
Ini::get( const wstring & section, const wstring & entry, CodePage cp ) const
{
	return narrow( get(section,entry), cp );
}


//-------------------------------------
//   function	Ini::findSection
//! @brief		Find section by name
//! @retval		(success) ... position of the section with specified name
//! @retval		(failure) ... empty string
//-------------------------------------
Ini::SectionIterator
Ini::findSection( const wstring & section )
{
	return FindSectionByName( m_Sections, section );
}


//-------------------------------------
//   function	Ini::erase
//! @brief		Delete the value of specified entry
//! @retval		(success) ... 0
//! @retval		(failure) ... -1
//-------------------------------------
short
Ini::erase( const string & section, const string & entry )
{
	return erase( widen(section), widen(entry) );
}

//-------------------------------------
//   function	Ini::erase
//! @brief		Delete the value of specified entry
//! @retval		(success) ... 0
//! @retval		(failure) ... -1
//-------------------------------------
short
Ini::erase( const wstring & section, const wstring & entry )
{
	EntryVec::iterator		entPos;
	SectionVec::iterator	secPos;
	EntryVec *				entries;

	// find specified section from all sections
	secPos = FindSectionByName( m_Sections, section );
	if( secPos == m_Sections.end() )
	{
		return -1; // not found
	}
	entries = &(secPos->entries);

	// find specified entry in the section
	entPos = FindEntryByName( *entries, entry );
	if( entPos == entries->end() )
	{
		return -1; // not found
	}

	// delete the entry
	entries->erase( entPos );
	
	// if the entry is the last entry of the section, delete the section
	if( entries->size() == 0 )
	{
		m_Sections.erase( secPos );
	}

	return 0;
}


//-------------------------------------
//   function	Ini::clear
//! @brief		Delete all data.
//-------------------------------------
void
Ini::clear()
{
	m_Sections.clear();
}


//-------------------------------------
//   function	Ini::begin
//! @brief		Get iterator pointing to the first section.
//-------------------------------------
Ini::SectionIterator
Ini::begin()
{
	return m_Sections.begin();
}

//-------------------------------------
//   function	Ini::begin
//! @brief		Get iterator pointing to the first section.
//-------------------------------------
Ini::ConstSectionIterator
Ini::begin() const
{
	return m_Sections.begin();
}


//-------------------------------------
//   function	Ini::end
//! @brief		Get termination node of vector containing section data.
//-------------------------------------
Ini::SectionIterator
Ini::end()
{
	return m_Sections.end();
}

//-------------------------------------
//   function	Ini::end
//! @brief		Get termination node of vector containing section data.
//-------------------------------------
Ini::ConstSectionIterator
Ini::end() const
{
	return m_Sections.end();
}


//-------------------------------------
//   function	Ini::load
//! @brief		open .ini file and parse it
//! @retval		(success) ... 0
//! @retval		(failure) ... -1
//-------------------------------------
short
Ini::load( const char * filePath, CodePage	cp )
{
	short	rc; // return code
	fstream	file;

	// open file
	file.open( filePath, fstream::in );
	if( file.fail() )
	{
		return -1;
	}

	// delegate to another method
	rc = load( file, cp );

	file.close();

	return rc;
}


//-------------------------------------
//   function	Ini::load
//! @brief		Read data from stream object
//! @returns	(success) ... 0
//				(failure) ... -1
//-------------------------------------
short
Ini::load( std::istream & stream, CodePage cp )
{
	int			rc; // return code
	string		line;
	wstring		entryName;
	wstring		entryValue;
	wstring		currentSectionName;
	Entry		entry;
	SectionIterator	secPos;
	EntryIterator	entPos;

	if( stream.bad() )
	{
		return -1;
	}

	// for each line of the file
	while( getline(stream,line) )
	{
		// check if this line is a section, and if so, parse it
		rc = ParseSectionLine( widen(line,cp), currentSectionName );
		if( rc == 0 )
		{
			continue; // go next line
		}

		// check if this line is an entry, and if so, parse it
		rc = ParseEntryLine( widen(line,cp), entryName, entryValue );
		if( rc == 0 )
		{
			// get current section
			secPos = FindSectionByName( m_Sections, currentSectionName );
			if( secPos == m_Sections.end() )
			{
				// the section not yet created. so create it
				Section	section;
				section.name = currentSectionName;
				m_Sections.push_back( section );
				secPos = find( m_Sections.begin(), m_Sections.end(), section );
				if( secPos == m_Sections.end() )
				{
					continue; // not found...? error. but ignore.
				}
			}

			// try to retrieve the entry
			entPos = FindEntryByName( secPos->entries, entryName );
			if( entPos != secPos->entries.end() )
			{
				// the entry already exists!
				// overwrite this entry with new value.
				entPos->value = entryValue;
				continue; // go next
			}

			// create entry node
			entry.name = entryName;
			entry.value = entryValue;

			// add entry to the section
			secPos->entries.push_back( entry );
		}
	}

	return 0;
}

//-------------------------------------
//   function	Ini::save
//! @brief		save to .ini file
//! @retval		(success) ... 0
//! @retval		(failure) ... -1
//-------------------------------------
short
Ini::save( const char * filePath, CodePage cp ) const
{
	short	rc; // return code
	fstream	file;

	// open file
	file.open( filePath );
	if( file.fail() )
	{
		return -1;
	}

	// delegate another method
	rc = save( file, cp );

	// close file
	file.close();

	return rc;
}


//-------------------------------------
//   function	Ini::save
//! @brief		save .ini formatted content to given stream
//! @retval		(success) ... 0
//! @retval		(failure) ... -1
//-------------------------------------
short
Ini::save( std::ostream & stream, CodePage cp ) const
{
	SectionVec::const_iterator	secPos;
	EntryVec::const_iterator	entPos;
	
	// validate stream to write in
	if( stream.bad() )
	{
		return -1;
	}

	// if this object has no data, nothing write.
	if( m_Sections.empty() )
	{
		return 0;
	}

	// write to file
	for( secPos=m_Sections.begin(); secPos!=m_Sections.end(); secPos++ )
	{
		// write section begin
		stream
			<< '['
			<< narrow( secPos->name, cp )
			<< ']' << endl;

		// write all sections in this section
		const EntryVec & entries = secPos->entries;
		for( entPos=entries.begin(); entPos!=entries.end(); entPos++ )
		{
			stream
				<< narrow( entPos->name, cp )
				<< '='
				<< narrow( entPos->value, cp ) << endl;
		}
	}

	return 0;
}

//-------------------------------------
//   function	Ini::narrow
//! @brief		Convert Unicode string to string in specified codepage.
//! @retval		(success) ... converted Sgry::wstring object
//! @retval		(failure) ... empty string
//-------------------------------------
std::string
Ini::narrow( const wstring & str, CodePage cp /*= Ini::UTF8*/ )
{
#	ifdef _WIN32
	int			rc; // return code
	char *		charBuf;
	long		neededBufSize;
	string		narrowStr;
	UINT		codepage;

	// convert code page to one of Win32API from one of Ini
	switch( cp )
	{
		case( Unicode ):
		{
			// cannot narrow...
			return string();
		}
		case Default:	codepage = CP_ACP;	break;
		default:		codepage = CP_UTF8;	break;
	}

	// get buffer size needed
	neededBufSize = WideCharToMultiByte( codepage, 0, (const WCHAR*)str.c_str(), (int)str.length(),
											NULL, 0, NULL, NULL );
	if( neededBufSize == 0 )
	{
		return string(); // invalid string for conversation?
	}

	// allocate buffer
	charBuf = new char[neededBufSize+1];

	// convert code page
	rc = WideCharToMultiByte( codepage, 0, (LPCWSTR)str.c_str(), (int)str.length(),
								charBuf, neededBufSize, NULL, NULL );
	if( rc == FALSE )
	{
		return string(); // failed to convert
	}
	charBuf[neededBufSize] = 0; // null terminate

	// make string
	narrowStr = charBuf;

	// delete buffer and return value
	delete charBuf;
	return narrowStr;
#	endif
}

//-------------------------------------
//   function	Ini::widen
//! @brief		Convert string in specified codepage to Unicode string
//! @returns	(success) ... converted Sgry::wstring object
//              (failure) ... empty string
//-------------------------------------
wstring
Ini::widen( const string & narrowStr, CodePage cp /*= Ini::UTF8*/ )
{
#	ifdef _WIN32
	int			rc; // return code
	wchar_t *	wcharBuf;
	long		neededBufSize;
	wstring		widenStr;
	UINT		codepage;

	// convert code page to one of Win32API from one of Ini
	switch( cp )
	{
		case( Unicode ):
		{
			// cannot narrow...
			return wstring();
		}
		case Default:	codepage = CP_ACP;	break;
		default:		codepage = CP_UTF8;	break;
	}

	// get buffer size needed
	neededBufSize = MultiByteToWideChar( codepage, 0, narrowStr.c_str(), (int)narrowStr.length(),
											NULL, 0 );
	if( neededBufSize == 0 )
	{
		return wstring(); // invalid string for conversation?
	}

	// allocate buffer
	wcharBuf = new wchar_t[neededBufSize+1];

	// convert code page
	rc = MultiByteToWideChar( codepage, 0, narrowStr.c_str(), (int)narrowStr.length(),
								(WCHAR*)wcharBuf, neededBufSize );
	if( rc == FALSE )
	{
		return wstring(); // failed to convert
	}
	wcharBuf[neededBufSize] = 0; // null terminate

	// make wstring
	widenStr = wstring( wcharBuf );

	// delete buffer and return value
	delete wcharBuf;
	return widenStr;
#	endif
}




//=============================================================================
//
//      section / entry object
//
//=============================================================================

//-------------------------------------
//   function	Ini::Section::begin
//! @brief		Get iterator pointing to the first entry of the section
//-------------------------------------
Ini::EntryIterator
Ini::Section::begin()
{
	return entries.begin();
}
Ini::ConstEntryIterator
Ini::Section::begin() const
{
	return entries.begin();
}


//-------------------------------------
//   function	Ini::Section::end
//! @brief		Get iterator pointing to the terminater node in entry-vector of the section
//-------------------------------------
Ini::EntryIterator
Ini::Section::end()
{
	return entries.end();
}
Ini::ConstEntryIterator
Ini::Section::end() const
{
	return entries.end();
}

//-------------------------------------
//   function	Ini::Section::operator ==
//-------------------------------------
bool
Ini::Section::operator == ( const Ini::Section & another )
{
	return (name == another.name);
}

//-------------------------------------
//   function	Ini::Entry::operator ==
//-------------------------------------
bool
Ini::Entry::operator == ( const Entry & another )
{
	return (name == another.name)&&(value == another.value);
}




//=============================================================================
//
//      Inner Functions
//
//=============================================================================

//-------------------------------------
// function : Ini::parseSectionLine
// brief    : parse 1 line as a section
// returns  : 0... success  /  -1 ... failure
//-------------------------------------
static short
ParseSectionLine( const wstring & line, wstring & outName )
{
	size_t	nameStartPos;
	size_t	nameEndPos;

	//---- extract section name ----//
	// detect start position of section name
	nameStartPos = line.find_first_of( L'[' );
	if( nameStartPos == -1 )
	{
		return -1; // invalid format
	}

	// detect end position of section name
	nameEndPos = line.find_last_of( L']' );
	if( nameEndPos == -1 )
	{
		return -1; // invalid format
	}

	// return section name
	outName = line.substr( nameStartPos+1, nameEndPos-1 );
	
	return 0;
}


//-------------------------------------
// function : Ini::parseEntryLine
// brief    : parse 1 line as an entry
// returns  : 0... success  /  -1 ... failure
//-------------------------------------
static short
ParseEntryLine( const wstring & line, wstring & outName, wstring & outValue )
{
	size_t	equalPos;

	// find equal char.
	equalPos = line.find( L'=' );
	if( equalPos == line.npos )
	{
		return -1;
	}

	// extract value
	outValue = line.substr( equalPos+1 );
	if( outValue.empty() )
	{
		return -1;
	}

	// trim value string
	outValue = Trim( outValue );
	if( outValue.empty() )
	{
		return -1;
	}

	// extract name
	outName = line.substr( 0, equalPos/*-0*/ );
	if( outName.empty() )
	{
		return -1;
	}

	// trim name string
	outName = Trim( outName );
	if( outName.empty() )
	{
		return -1;
	}

	return 0;
}


//-------------------------------------
// function : Trim
// brief    : Trim white spaces at start and at end.
//-------------------------------------
static wstring
Trim( wstring & str )
{
	size_t	beginPos;
	size_t	endPos;

	// get position of first non white-space char.
	beginPos = 0;
	while( IsWhiteSpaceChar(str[beginPos]) )
	{
		// reached to the end?
		if( beginPos == str.length() )
		{
			return wstring(); // all charactors were white-space.
		}

		beginPos++;
	}

	// get position of last non white-space char.
	endPos = str.length() - 1;
	while( IsWhiteSpaceChar(str[endPos]) )
	{
		// reached to the begin?
		if( endPos == 0 )
		{
			return wstring(); // all charactors were white-space.
		}

		endPos--;
	}
	
	// validate indexes
	if( endPos < beginPos )
	{
		return wstring();
	}
	
	// return value
	return str.substr( beginPos, endPos+1 );
}


//-------------------------------------
// function : FindSectionByName
// brief    : Find section having the given name and returns its position
// note     : if failed, returns nodes.end().
//-------------------------------------
static inline SectionVec::iterator
FindSectionByName( SectionVec & sections, const wstring & name )
{
	SectionVec::iterator	secPos;

	// seek through all nodes
	for( secPos=sections.begin(); secPos!=sections.end(); secPos++ )
	{
		// if the node has same name, return its position.
		if( secPos->name == name )
		{
			return secPos;
		}
	}

	return secPos;
}
static inline SectionVec::const_iterator
FindSectionByName( const SectionVec & sections, const wstring & name )
{
	SectionVec::const_iterator	secPos;

	// seek through all nodes
	for( secPos=sections.begin(); secPos!=sections.end(); secPos++ )
	{
		// if the node has same name, return its position.
		if( secPos->name == name )
		{
			return secPos;
		}
	}

	return secPos;
}


//-------------------------------------
// function : FindEntryByName
// brief    : Get position of an entry having the given name
// note     : if failed, returns sections.end().
//-------------------------------------
static inline EntryVec::iterator
FindEntryByName( EntryVec & entries, const wstring & name )
{
	EntryVec::iterator	entPos;

	// seek through all nodes
	for( entPos=entries.begin(); entPos!=entries.end(); entPos++ )
	{
		// if the node has same name, return its position.
		if( entPos->name == name )
		{
			return entPos;
		}
	}

	return entPos;
}
static inline EntryVec::const_iterator
FindEntryByName( const EntryVec & entries, const wstring & name )
{
	EntryVec::const_iterator	entPos;

	// seek through all nodes
	for( entPos=entries.begin(); entPos!=entries.end(); entPos++ )
	{
		// if the node has same name, return its position.
		if( entPos->name == name )
		{
			return entPos;
		}
	}

	return entPos;
}

/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
