/*! @file		Ini.h
 *  @brief		".ini" file manipulation class
 *  @date		2007-05-57 (SGRY)
 */
/*  dependency	none
 *  license		MIT License (see end of this file)
 ***********************************************/
#pragma once

// prevent hundreds of warnings from Visual C++ 6.0
#ifdef _MSC_VER
#	pragma warning( disable : 4097 ) // 'restore' or 'off' is needed for pragma parameter
#	pragma warning( disable : 4127 ) // condition is constant
#	pragma warning( disable : 4503 ) // too long identifier
#	pragma warning( disable : 4512 ) // cannot to generate operator =
#	pragma warning( disable : 4514 ) // unreferenced inline function deleted
#	pragma warning( disable : 4786 ) // too long identifier truncated (over 255)
#endif

#include <fstream>
#include <sstream>
#include <string>
#include <vector>


// include all into my original namespace
namespace Sgry {

#ifndef _SGRY_STRING_DEFINED_
#define _SGRY_STRING_DEFINED_
	typedef std::basic_string<wchar_t>		wstring;
	typedef std::basic_stringstream<wchar_t>	wstringstream;
	typedef std::basic_string<char>			string;
	typedef std::basic_stringstream<char>	stringstream;
#endif

/*!
 * @class	Ini
 * @brief	Class for manipulating .ini format file.
 */
class Ini
{
	// inner data types (forward declaration)
	public:
		class Entry;
		class Section;

		//! iterator for entries
		typedef std::vector<Entry>::iterator
			EntryIterator;

		//! readonly iterator for entries
		typedef std::vector<Entry>::const_iterator
			ConstEntryIterator;

		//! iterator for sections
		typedef std::vector<Section>::iterator
			SectionIterator;

		//! readonly iterator for sections
		typedef std::vector<Section>::const_iterator
			ConstSectionIterator;

		//! Flags for specifying code page
		enum CodePage
		{
			Default	= 0,	//!< System default code page
			Unicode,		//!< Unicode (UTF16) code page
			UTF8			//!< UTF-8 code page
		};

	// public methods
	public:
		Ini();

		wstring			get( const string & section, const string & entry ) const;
		wstring			get( const wstring & section, const wstring & entry ) const;
		string			get( const wstring & section, const wstring & entry, CodePage cp ) const;
		short			set( const string & section, const string & entry, const wstring & value );
		short			set( const wstring & section, const wstring & entry, const wstring & value );
		short			set( const wstring & section, const wstring & entry, const string & value, CodePage cp );
		short			erase( const string & section, const string & entry );
		short			erase( const wstring & section, const wstring & entry );
		void			clear();

		SectionIterator			begin();
		ConstSectionIterator	begin() const;
		SectionIterator			end();
		ConstSectionIterator	end() const;
		SectionIterator			findSection( const wstring & section );

		short		load( const char * filePath, CodePage cp = UTF8 );
		short		load( std::istream & stream, CodePage cp = UTF8 );
		short		save( const char * filePath, CodePage cp = UTF8 ) const;
		short		save( std::ostream & stream, CodePage cp = UTF8 ) const;

		static string	narrow( const wstring & str, CodePage cp = UTF8 );
		static wstring	widen( const string & str, CodePage cp = UTF8 );

	// inner data types
	public:
		//! expresses an entry of .ini format
		class Entry
		{
			public:
				wstring	name;		//!< entry name
				wstring	value;		//!< value of an entry
				bool operator == ( const Entry & another );
		};

		//! expresses a section of .ini format
		class Section
		{
			public:
				wstring				name;				//!< section name
				std::vector<Entry>	entries;			//!< entry nodes included in the section
				bool operator == ( const Section & another );
				EntryIterator		begin();
				ConstEntryIterator	begin() const;
				EntryIterator		end();
				ConstEntryIterator	end() const;
		};

	// glue methods
	public:
		inline short
		set( const wchar_t * section, const wchar_t * entry, const char * value, CodePage cp )
		{
			return set( wstring(section), wstring(entry), string(value), cp );
		}
		inline short
		erase( const wchar_t * section, const wchar_t * entry )
		{
			return erase( wstring(section), wstring(entry) );
		}
		inline short
		save( const wchar_t * filePath, CodePage cp = UTF8 ) const
		{
			return save( narrow(filePath,Default).c_str(), cp );
		}
		inline short
		load( const wchar_t * filePath, CodePage cp = UTF8 )
		{
			return load( narrow(filePath,Default).c_str(), cp );
		}

	// attributes
	private:
		std::vector<Section>	m_Sections;
};


}; // namespace Sgry

/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
