// file    : FocusTalk.cpp
// brief   : class to handle FocusTalk as a speaker
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-10-10 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "Speakers/FocusTalk.h"
#include "Utilities.h"
#	include <assert.h>

const int FT_OUTPUT_ABSOLUTE = 0x01;
const int FT_OUTPUT_KEYDOWN = 0x20;

//=============================================================================
//
//      Init / Dispose
//
//=============================================================================

//-------------------------------------
// function : FocusTalk::FocusTalk
// brief    : initialize object
//-------------------------------------
FocusTalk::FocusTalk()
{
	_Dll = NULL;
	FT_OutputVoiceText = NULL;
	FT_StopSound = NULL;
	FT_IsSoundBusy = NULL;
	FT_ChangeVoiceType = NULL;
	FT_GetVoiceType = NULL;
}

//-------------------------------------
// function : FocusTalk::~FocusTalk
// brief    : dispose object
//-------------------------------------
FocusTalk::~FocusTalk()
{
	if( _Dll != NULL )
	{
		FreeLibrary( _Dll );
	}
}


//-------------------------------------
// function : FocusTalk::Init
// brief    : do preparation to use FocusTalk as the speaker
// returns  : (success) ... 0
//            (failure) ... -1
//-------------------------------------
int
FocusTalk::Init()
{
	// try to load API DLL
	_Dll = LoadLibraryA( "FocusTalkExt.dll" );
	if( _Dll == NULL )
	{
		goto cleanup;
	}

	// get the function to make FocusTalk speak
	FT_OutputVoiceText = (FT_OutputVoiceTextProc)GetProcAddress( _Dll, "FT_OutputVoiceText" );
	if( FT_OutputVoiceText == NULL )
	{
		goto cleanup;
	}
	FT_StopSound = (FT_StopSoundProc)GetProcAddress( _Dll, "FT_StopSound" );
	if( FT_StopSound == NULL )
	{
		goto cleanup;
	}
	FT_IsSoundBusy = (FT_IsSoundBusyProc)GetProcAddress( _Dll, "FT_IsSoundBusy" );
	if( FT_IsSoundBusy == NULL )
	{
		goto cleanup;
	}
	FT_ChangeVoiceType = (FT_ChangeVoiceTypeProc)GetProcAddress( _Dll, "FT_ChangeVoiceType" ); // 2.0.2 or later only
	FT_GetVoiceType = (FT_GetVoiceTypeProc)GetProcAddress( _Dll, "FT_GetVoiceType" ); // 2.0.2 or later only

	// try to manipulate FocusTalk
	if( FT_OutputVoiceText("", FT_OUTPUT_ABSOLUTE) == FALSE )
	{
		// probably FocusTalk is not launched.
		goto cleanup;
	}

	return 0;
cleanup:
	FT_OutputVoiceText = NULL;
	FT_StopSound = NULL;
	FT_IsSoundBusy = NULL;
	FT_ChangeVoiceType = NULL;
	FT_GetVoiceType = NULL;
	return -1;
}

//-------------------------------------
// function : FocusTalk::Speak
// brief    : speak text
//-------------------------------------
void
FocusTalk::Speak( const wchar_t * str )
{
	FT_OutputVoiceText( Utl::Narrow(str).c_str(), FT_OUTPUT_ABSOLUTE );
}

//-------------------------------------
// function : FocusTalk::SpeakLow
// brief    : speak text with low pitch voice
//-------------------------------------
void
FocusTalk::SpeakLow( const wchar_t * str )
{
	int voiceType, subVoiceType;
	
	if( FT_ChangeVoiceType != NULL )
	{
		// ݂̐ɉĕ̐
		voiceType = FT_GetVoiceType();
		if( voiceType == 0 )
			subVoiceType = 1;
		else
			subVoiceType = 0;

		// 𔭐
		FT_ChangeVoiceType( subVoiceType );
		FT_OutputVoiceText( Utl::Narrow(str).c_str(), 0 );
		FT_ChangeVoiceType( voiceType );
	}
	else
	{
		// o[W2.0.2ȍ~łȂB
		// 剹ŔBŒɂB
		FT_OutputVoiceText( Utl::Narrow(str).c_str(), FT_OUTPUT_ABSOLUTE|FT_OUTPUT_KEYDOWN );
	}
}

//-------------------------------------
// function : FocusTalk::SpeakAndWait
// brief    : speak text and wait until it ends
//-------------------------------------
void
FocusTalk::SpeakAndWait( const wchar_t * str )
{
	Speak( str );

	// wait until the speech ends
	while( FT_IsSoundBusy() == TRUE )
	{
		Sleep( 50 );
	}
}

//-------------------------------------
// function : FocusTalk::Wait
// brief    : add wait to speech buffer
// params   : waitLong ... wait duration will be longer if true
//-------------------------------------
void
FocusTalk::Wait( bool waitLong )
{
	waitLong;
	// Calling SoundMessage in series does not work well.
	// So because calling SoundMessage in this function affects whole part of the program,
	// temporarily disable this function.
	/*
	char kuten[3], touten[3];

	// SoundMessage without PF_NATURAL_PROOF flag do not read any symbols
	int flag = PF_DONT_IGNORE;

	// read punctuation to wait a moment
	if( waitLong )
	{
		// set Kuten in Japanese
		kuten[0] = '\x81';
		kuten[1] = '\x42';
		kuten[2] = '\x00';
		SoundMessage( kuten, flag );
	}
	else
	{
		// set Touten in Japanese
		touten[0] = '\x81';
		touten[1] = '\x41';
		touten[2] = '\x00';
		SoundMessage( touten, flag );
	}
	*/
}

//-------------------------------------
// function : FocusTalk::ShutUp
// brief    : stop speech immediately
//-------------------------------------
void
FocusTalk::ShutUp()
{
	FT_StopSound();
}

//-------------------------------------
// function : FocusTalk::GetTextForTripleDots
// brief    : get text that would be read when SR reads a menu item with "..."
//-------------------------------------
wstring
FocusTalk::GetTextForTripleDots()
{
	wchar_t text[7];

	// set "pi ltu pi ltu pi ltu" in Japanese
	text[0] = 0x30d4;
	text[1] = 0x30c3;
	text[2] = 0x30d4;
	text[3] = 0x30c3;
	text[4] = 0x30d4;
	text[5] = 0x30c3;
	text[6] = 0;

	return wstring( text );
}

/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
