// file    : Jaws.cpp
// brief   : class to handle Jaws 6.2/7.1 as a speaker
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-06-15 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
// TODO: if JAWS 6 and 7 is installed and 6 is running, JAWS dir would be set as 6's dir
#include "Speakers/Jaws.h"
#include "Utilities.h"

inline static string GetJawsDir();

//=============================================================================
//
//      Init / Dispose
//
//=============================================================================

//-------------------------------------
// function : Jaws::Jaws
// brief    : initialize object
//-------------------------------------
Jaws::Jaws()
{
	_Dll = NULL;
	JFWSayString = NULL;
	JFWStopSpeech = NULL;
}

//-------------------------------------
// function : Jaws::~Jaws
// brief    : dispose object
//-------------------------------------
Jaws::~Jaws()
{
	if( _Dll != NULL )
	{
		FreeLibrary( _Dll );
	}
}

//-------------------------------------
// function : Jaws::Init
// brief    : do preparation to use Jaws as the speaker
// returns  : (success) ... 0
//            (failure) ... -1
//-------------------------------------
int
Jaws::Init()
{
	int		rc; // result code
	string	dllPath;

	//---- locate JAWS API DLL ----
	dllPath = GetJawsDir();
	if( dllPath.empty() )
	{
		goto error; // no supported JAWS is installed
	}
	dllPath += "\\jfwapi.dll";

	//---- load API functions ----
	_Dll = LoadLibraryA( dllPath.c_str() );
	if( _Dll == NULL )
	{
		goto error; // installation broken?
	}

	// load "JFWSayString"
	JFWSayString = (JFWSayStringProc)GetProcAddress( _Dll, "JFWSayString" );
	if( JFWSayString == NULL )
	{
		goto error;
	}

	// load "JFWStopSpeech"
	JFWStopSpeech = (JFWStopSpeechProc)GetProcAddress( _Dll, "JFWStopSpeech" );
	if( JFWStopSpeech == NULL )
	{
		goto error;
	}

	//---- ensure JAWS is running ----
	rc = JFWSayString( "", FALSE );
	if( rc == FALSE )
	{
		goto error; // not running
	}

	return 0;
error:
	_Dll = NULL;
	JFWSayString = NULL;
	JFWStopSpeech = NULL;
	return -1;
}

//-------------------------------------
// function : Jaws::Speak
// brief    : speak text
//-------------------------------------
void
Jaws::Speak( const wchar_t * str )
{
	JFWSayString( Utl::Narrow(str).c_str(), FALSE );
}

//-------------------------------------
// function : Jaws::SpeakLow
// brief    : speak text with low pitch voice
//-------------------------------------
void
Jaws::SpeakLow( const wchar_t * str )
{
	// set voice pitch low
	;

	// speak
	JFWSayString( Utl::Narrow(str).c_str(), FALSE );

	// restore voice pitch
	;
}

//-------------------------------------
// function : Jaws::SpeakAndWait
// brief    : speak text and wait until it ends
//-------------------------------------
void
Jaws::SpeakAndWait( const wchar_t * str )
{
	Speak( str );
}

//-------------------------------------
// function : Jaws::Wait
// brief    : add wait to speech buffer
// params   : waitLong ... wait duration will be longer if true
//-------------------------------------
void
Jaws::Wait( bool waitLong )
{
	if( waitLong )
		JFWSayString( "B", FALSE );
	else
		JFWSayString( "A", FALSE );
}

//-------------------------------------
// function : Jaws::ShutUp
// brief    : stop speech immediately
//-------------------------------------
void
Jaws::ShutUp()
{
	JFWStopSpeech();
}

//-------------------------------------
// function : Jaws::GetTextForTripleDots
// brief    : get text that would be read when SR reads a menu item with "..."
//-------------------------------------
wstring
Jaws::GetTextForTripleDots()
{
	return wstring( L"..." );
}


//-------------------------------------
// function : GetJawsDir
// returns  : (success) ... latest JAWS installed directory
//            (failure) ... empty string
// notes    : currently this supports 6.20 and 7.10.
//-------------------------------------
inline static string
GetJawsDir()
{
	string jawsDir;

	// key : HKEY_LOCAL_MACHINE\SOFTWARE\Freedom Scientific\JAWS\6.20
	jawsDir = Sgry::Reg_ReadStringA( HKEY_LOCAL_MACHINE, "SOFTWARE\\Freedom Scientific\\JAWS\\6.20", "Target" );
	if( jawsDir.empty() )
	{
		return string();
	}

	// key : HKEY_LOCAL_MACHINE\SOFTWARE\Freedom Scientific\JAWS\7.10
	jawsDir = Sgry::Reg_ReadStringA( HKEY_LOCAL_MACHINE, "SOFTWARE\\Freedom Scientific\\JAWS\\7.10", "Target" );
	if( jawsDir.empty() )
	{
		return string();
	}

	return jawsDir;
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
