// file    : PCTalker.cpp
// brief   : class to handle PC-Talker as a speaker
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-06-15 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "Speakers/PCTalker.h"
#include "Utilities.h"

//=============================================================================
//
//      Init / Dispose
//
//=============================================================================

//-------------------------------------
// function : PCTalker::PCTalker
// brief    : initialize object
//-------------------------------------
PCTalker::PCTalker()
{
	_Dll = NULL;
	PCTKPRead = NULL;
	PCTKStatus = NULL;
	PCTKVReset = NULL;
	PCTKGetStatus = NULL;
	PCTKSetStatus = NULL;
	SoundStatus = NULL;
}

//-------------------------------------
// function : PCTalker::~PCTalker
// brief    : dispose object
//-------------------------------------
PCTalker::~PCTalker()
{
	if( _Dll != NULL )
	{
		FreeLibrary( _Dll );
	}
}

//-------------------------------------
// function : PCTalker::Init
// brief    : do preparation to use PC-Talker as the speaker
// returns  : (success) ... 0
//            (failure) ... -1
//-------------------------------------
int
PCTalker::Init()
{
	// try to load API DLL
	_Dll = LoadLibraryA( "PCTKUSR.dll" );
	if( _Dll == NULL )
	{
		goto cleanup;
	}

	// PC-Talker was installed to this system.
	// get a function from DLL which can check whether PC-Talker is launched or not.
	PCTKStatus = (PCTKStatusProc)GetProcAddress( _Dll, "PCTKStatus" );
	if( PCTKStatus == NULL )
	{
		goto cleanup;
	}

	// PC-Talker launched now?
	if( PCTKStatus() != TRUE )
	{
		goto cleanup;
	}

	// get the function to make PC-Talker speak
	PCTKPRead = (PCTKPReadProc)GetProcAddress( _Dll, "PCTKPRead" );
	if( PCTKPRead == NULL )
	{
		goto cleanup;
	}
	PCTKVReset = (PCTKVResetProc)GetProcAddress( _Dll, "PCTKVReset" );
	if( PCTKPRead == NULL )
	{
		goto cleanup;
	}

	// get other functions
	PCTKGetStatus = (PCTKGetStatusProc)GetProcAddress( _Dll, "PCTKGetStatus" );
	if( PCTKGetStatus == NULL )
	{
		goto cleanup;
	}
	PCTKSetStatus = (PCTKSetStatusProc)GetProcAddress( _Dll, "PCTKSetStatus" );
	if( PCTKSetStatus == NULL )
	{
		goto cleanup;
	}
	SoundStatus = (SoundStatusProc)GetProcAddress( _Dll, "SoundStatus" );
	if( SoundStatus == NULL )
	{
		goto cleanup;
	}

	return 0;
cleanup:
	_Dll = NULL;
	PCTKStatus = NULL;
	PCTKPRead = NULL;
	PCTKVReset = NULL;
	PCTKGetStatus = NULL;
	PCTKSetStatus = NULL;
	SoundStatus = NULL;
	return -1;
}


//-------------------------------------
// function : PCTalker::Speak
// brief    : speak text
//-------------------------------------
void
PCTalker::Speak( const wchar_t * str )
{
	// modify non CR+LF EOL code to CR+LF
	if( wcslen(str) == 1
		&& (str[0] == '\r' || str[0] == '\n') )
	{
		PCTKPRead( "\r\n", 5, TRUE );
		return;
	}

	// do speech
	PCTKPRead( Utl::Narrow(str).c_str(), 5, TRUE );
}

//-------------------------------------
// function : PCTalker::SpeakLow
// brief    : speak text with low pitch voice
//-------------------------------------
void
PCTalker::SpeakLow( const wchar_t * str )
{
	const int PS_STATUS = 0x00010001;
	const INT_PTR PKDI_RPITCH = 24;
	DWORD orgPitch;
	
	// keep original voice pitch and then set lowest
	orgPitch = PCTKGetStatus( PS_STATUS, (void*)PKDI_RPITCH, NULL );
	PCTKSetStatus( PS_STATUS, (void*)PKDI_RPITCH, (void*)0 );

	// speak with low pitch voice
	PCTKPRead( Utl::Narrow(str).c_str(), 5, FALSE );

	// restore voice pitch
	PCTKSetStatus( PS_STATUS, (void*)PKDI_RPITCH, (void*)(INT_PTR)orgPitch );
}

//-------------------------------------
// function : PCTalker::SpeakAndWait
// brief    : speak text and wait until it ends
//-------------------------------------
void
PCTalker::SpeakAndWait( const wchar_t * str )
{
	Speak( str );

	// wait until the speech ends
	while( SoundStatus() == TRUE )
	{
		Sleep( 50 );
	}
}

//-------------------------------------
// function : PCTalker::Wait
// brief    : add wait to speech buffer
// params   : waitLong ... wait duration will be longer if true
//-------------------------------------
void
PCTalker::Wait( bool waitLong )
{
	const int PS_STATUS = 0x00010001;
	const INT_PTR PKDI_KUTEN = 17;
	char kuten[3], touten[3];
	
	// disable reading punctuation
	DWORD readsKuten = PCTKGetStatus( PS_STATUS, (void*)PKDI_KUTEN, NULL );
	PCTKSetStatus( PS_STATUS, (void*)PKDI_KUTEN, (void*)0 );

	// read punctuation to wait a moment
	if( waitLong )
	{
		// set Kuten in Japanese
		kuten[0] = '\x81';
		kuten[1] = '\x42';
		kuten[2] = '\x00';
		PCTKPRead( kuten, 5, FALSE );
	}
	else
	{
		// set Touten in Japanese
		touten[0] = '\x81';
		touten[1] = '\x41';
		touten[2] = '\x00';
		PCTKPRead( touten, 5, FALSE );
	}

	// restore setting of reading punctuation
	PCTKSetStatus( PS_STATUS, (void*)PKDI_KUTEN, (void*)(INT_PTR)readsKuten );
}

//-------------------------------------
// function : PCTalker::ShutUp
// brief    : stop speech immediately
//-------------------------------------
void
PCTalker::ShutUp()
{
	PCTKVReset();
}

//-------------------------------------
// function : PCTalker::GetTextForNewWindowMenuItem
// brief    : get text that would be read when SR reads a menu item with "..."
//-------------------------------------
wstring
PCTalker::GetTextForTripleDots()
{
	wchar_t text[6];

	// set "window" in Japanese
	text[0] = 0x30a6;
	text[1] = 0x30a3;
	text[2] = 0x30f3;
	text[3] = 0x30c9;
	text[4] = 0x30a6;
	text[5] = 0;

	return wstring( text );
}

/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
