// file    : XPReader.cpp
// brief   : class to handle XPReader as a speaker
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-06-15 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
#include "Speakers/XPReader.h"
#include "Utilities.h"

const int PF_DONT_IGNORE = 2;
const int PF_TONE_DOWN = 0x20; // speak with low pitch voice
const int PF_NATURAL_PROOF = 0x40; // let XPReader preprocess

//=============================================================================
//
//      Init / Dispose
//
//=============================================================================

//-------------------------------------
// function : XPReader::XPReader
// brief    : initialize object
//-------------------------------------
XPReader::XPReader()
{
	_Dll = NULL;
	SoundMessage = NULL;
	SoundStop = NULL;
	SoundStatus = NULL;
	SoundModifyMode = NULL;
}

//-------------------------------------
// function : XPReader::~XPReader
// brief    : dispose object
//-------------------------------------
XPReader::~XPReader()
{
	if( _Dll != NULL )
	{
		FreeLibrary( _Dll );
	}
}


//-------------------------------------
// function : XPReader::Init
// brief    : do preparation to use XPReader as the speaker
// returns  : (success) ... 0
//            (failure) ... -1
//-------------------------------------
int
XPReader::Init()
{
	// try to load API DLL
	_Dll = LoadLibraryA( "SOUNDP.dll" );
	if( _Dll == NULL )
	{
		goto cleanup;
	}

	// XPReader was installed to this system.
	// then, check the XPReader window exists
	if( FindWindow("95Reader",0) == 0 )
	{
		goto cleanup;
	}

	// get the function to make XPReader speak
	SoundMessage = (SoundMessageProc)GetProcAddress( _Dll, "SoundMessage" );
	if( SoundMessage == NULL )
	{
		goto cleanup;
	}
	SoundStop = (SoundStopProc)GetProcAddress( _Dll, "SoundStop" );
	if( SoundStop == NULL )
	{
		goto cleanup;
	}
	SoundStatus = (SoundStatusProc)GetProcAddress( _Dll, "SoundStatus" );
	if( SoundStatus == NULL )
	{
		goto cleanup;
	}
	SoundModifyMode = (SoundModifyModeProc)GetProcAddress( _Dll, "SoundModifyMode" );
	if( SoundModifyMode == NULL )
	{
		goto cleanup;
	}

	return 0;
cleanup:
	SoundMessage = NULL;
	SoundStop = NULL;
	SoundStatus = NULL;
	SoundModifyMode = NULL;
	return -1;
}

//-------------------------------------
// function : XPReader::Speak
// brief    : speak text
//-------------------------------------
void
XPReader::Speak( const wchar_t * str )
{
	int flag = PF_DONT_IGNORE | PF_NATURAL_PROOF;
	SoundMessage( Utl::Narrow(str).c_str(), flag );
}

//-------------------------------------
// function : XPReader::SpeakLow
// brief    : speak text with low pitch voice
//-------------------------------------
void
XPReader::SpeakLow( const wchar_t * str )
{
	int flag = PF_DONT_IGNORE | PF_TONE_DOWN | PF_NATURAL_PROOF;
	SoundMessage( Utl::Narrow(str).c_str(), flag );
}

//-------------------------------------
// function : XPReader::SpeakAndWait
// brief    : speak text and wait until it ends
//-------------------------------------
void
XPReader::SpeakAndWait( const wchar_t * str )
{
	Speak( str );

	// wait until the speech ends
	while( SoundStatus() == TRUE )
	{
		Sleep( 50 );
	}
}

//-------------------------------------
// function : XPReader::Wait
// brief    : add wait to speech buffer
// params   : waitLong ... wait duration will be longer if true
//-------------------------------------
void
XPReader::Wait( bool waitLong )
{
	waitLong;
	// Calling SoundMessage in series does not work well.
	// So because calling SoundMessage in this function affects whole part of the program,
	// temporarily disable this function.
	/*
	char kuten[3], touten[3];

	// SoundMessage without PF_NATURAL_PROOF flag do not read any symbols
	int flag = PF_DONT_IGNORE;

	// read punctuation to wait a moment
	if( waitLong )
	{
		// set Kuten in Japanese
		kuten[0] = '\x81';
		kuten[1] = '\x42';
		kuten[2] = '\x00';
		SoundMessage( kuten, flag );
	}
	else
	{
		// set Touten in Japanese
		touten[0] = '\x81';
		touten[1] = '\x41';
		touten[2] = '\x00';
		SoundMessage( touten, flag );
	}
	*/
}

//-------------------------------------
// function : XPReader::ShutUp
// brief    : stop speech immediately
//-------------------------------------
void
XPReader::ShutUp()
{
	SoundStop();
}

//-------------------------------------
// function : XPReader::GetTextForTripleDots
// brief    : get text that would be read when SR reads a menu item with "..."
//-------------------------------------
wstring
XPReader::GetTextForTripleDots()
{
	wchar_t text[7];

	// set "pi ltu pi ltu pi ltu" in Japanese
	text[0] = 0x30d4;
	text[1] = 0x30c3;
	text[2] = 0x30d4;
	text[3] = 0x30c3;
	text[4] = 0x30d4;
	text[5] = 0x30c3;
	text[6] = 0;

	return wstring( text );
}

/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
