// file    : SwingEventServer.cpp
// brief   : defines a internal server that serves Swing event sent from AccessBridge
// author  : SGRY (YAMAMOTO Suguru)
// update  : 2007-10-08 (SGRY)
// license : MIT License (see END of this file)
//=========================================================
//#define TRACE_EVENT

#include <iostream>
#include "SwingEventServer.h"

#include "Speakers/FocusTalk.h"
#include "Speakers/PCTalker.h"
#include "Speakers/XPReader.h"
#include "Speakers/Jaws.h"
#include "Speakers/ConsoleSpeaker.h"

#include "ControlReaders/GenericControlReader.h"
#include "ControlReaders/ListReader.h"
#include "ControlReaders/TextReader.h"
#include "ControlReaders/ButtonReader.h"
#include "ControlReaders/ToggleButtonReader.h"
#include "ControlReaders/CheckBoxReader.h"
#include "ControlReaders/ComboBoxReader.h"
#include "ControlReaders/MenuReader.h"
#include "ControlReaders/MenuItemReader.h"
#include "ControlReaders/RadioButtonReader.h"
#include "ControlReaders/TreeReader.h"

using namespace std;

// (instantiate static fields in SwingEventServer class)
SwingEventServer::ControlReaders SwingEventServer::_ControlReaders;
bool SwingEventServer::_InstanceCreated = false;

// static functions
inline static void TRACE_PRINT( const char * format, ... );

//=============================================================================
//
//   Init / Dispose
//
//=============================================================================

//-------------------------------------
// function : SwingEventServer::SwingEventServer
// brief    : create new instance
//-------------------------------------
SwingEventServer::SwingEventServer()
{
	_Speaker = NULL;
}


//-------------------------------------
// function : SwingEventServer::~SwingEventServer
// brief    : dispose resources
//-------------------------------------
SwingEventServer::~SwingEventServer()
{
	// delete speaker object
	delete _Speaker;

	// delete all control readers
	for( size_t i=0; i<_ControlReaders.size(); i++ )
	{
		if( _ControlReaders[i] != NULL )
		{
			delete _ControlReaders[i];
			_ControlReaders[i] = NULL;
		}
	}

	// uninstall event handlers
	SetJavaShutdown( NULL );
	SetFocusGained( NULL );
	SetCaretUpdate( NULL );
	SetMenuSelected( NULL );
	SetPropertyCaretChange( NULL );
	SetPropertyStateChange( NULL );
	SetPropertySelectionChange( NULL );
	SetPropertyActiveDescendentChange( NULL );
	
	_InstanceCreated = false;
}

//-------------------------------------
// function : SwingEventServer::Init
// brief    : initialize control readers and ready to serve Swing events
//-------------------------------------
int
SwingEventServer::Init()
{
	// initialize fields
	_Speaker = CreateSpeaker();
	if( _Speaker == NULL )
	{
		return -1;
	}
	_InstanceCreated = true;

	// create control readers and be ready for handling Swing events
	// (note: combo-box has a list inside so list reader should receive events before combo reader)
	_ControlReaders.push_back( new ButtonReader(_Speaker) );
	_ControlReaders.push_back( new RadioButtonReader(_Speaker) );
	_ControlReaders.push_back( new CheckBoxReader(_Speaker) );
	_ControlReaders.push_back( new ToggleButtonReader(_Speaker) );
	_ControlReaders.push_back( new ListReader(_Speaker) );
	_ControlReaders.push_back( new MenuReader(_Speaker) );
	_ControlReaders.push_back( new MenuItemReader(_Speaker) );
	_ControlReaders.push_back( new TextReader(_Speaker) );
	_ControlReaders.push_back( new TreeReader(_Speaker) );
	_ControlReaders.push_back( new ComboBoxReader(_Speaker) );
	_ControlReaders.push_back( new GenericControlReader(_Speaker) );

	// install event handlers
	SetJavaShutdown( &SwingEventServer::OnJavaShutdown );
	SetFocusGained( &SwingEventServer::OnFocusGained );
	SetCaretUpdate( &SwingEventServer::OnCaretUpdate );
	//DONT//SetMenuDeselected( &SwingEventServer::OnMenuDeselected );
	//DONT//SetMenuCanceled( &SwingEventServer::OnMenuCanceled );
	SetMenuSelected( &SwingEventServer::OnMenuSelected );
	SetPropertyCaretChange( &SwingEventServer::OnCaretPropertyChange );
	SetPropertyStateChange( &SwingEventServer::OnStatePropertyChange );
	SetPropertySelectionChange( &SwingEventServer::OnSelectionPropertyChange );
	SetPropertyActiveDescendentChange( &SwingEventServer::OnActiveDescendentPropertyChange );

	return 0;
}

//-------------------------------------
// function : SwingEventServer::CreateSpeaker
// brief    : create proper speaker object for current environment
//-------------------------------------
ISpeakable *
SwingEventServer::CreateSpeaker()
{
	int rc; // result code
	FocusTalk * ft;
	PCTalker * pcTalker;
	XPReader * xpReader;
	Jaws * jaws;
	ConsoleWriter * console;

	// try to use FocusTalk as the speaker
	ft = new FocusTalk();
	rc = ft->Init();
	if( rc == 0 )
	{
		return ft; // okay to use FocusTalk
	}
	delete ft;

	// try to use PC-Talker as the speaker
	pcTalker = new PCTalker();
	rc = pcTalker->Init();
	if( rc == 0 )
	{
		return pcTalker; // okay to use PC-Talker
	}
	delete pcTalker;

	// try to use XPReader as the speaker
	xpReader = new XPReader();
	rc = xpReader->Init();
	if( rc == 0 )
	{
		return xpReader; // okay to use XPReader
	}
	delete xpReader;

	// try to use JAWS as the speaker
	jaws = new Jaws();
	rc = jaws->Init();
	if( rc == 0 )
	{
		return jaws; // okay to use JAWS
	}
	delete jaws;

	// finally, use console as the speaker; simply display string
	console = new ConsoleWriter();
	rc = console->Init();
	if( rc == 0 )
	{
		return console;
	}
	delete console;

	return NULL;
}

//=============================================================================
//
//   Event handlers
//
//=============================================================================

//-------------------------------------
// function : SwingEventServer::OnFocusGained
// brief    : notifies all control readers that a Swing component gained the focus
//-------------------------------------
void
SwingEventServer::OnFocusGained( long vmID, FocusEvent event, AccessibleContext context )
{
	BOOL rc; // result code
	AccessibleContextInfo contextInfo;
	ControlReaders::iterator it;

	// get context information
	rc = GetAccessibleContextInfo( vmID, context, &contextInfo );
	if( rc == FALSE )
	{
		goto cleanup;
	}

	TRACE_PRINT("OnFocusGained (%s / event:%X)\n", Utl::Narrow(contextInfo.name).c_str(), event);

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		if( reader->CanHandleContext(vmID, context, contextInfo) )
		{
			reader->OnFocusGained( vmID, event, context, contextInfo );
			break; // only one reader can handle event
		}
	}

cleanup:
	// dispose Java objects
	ReleaseJavaObject( vmID, event );
	ReleaseJavaObject( vmID, context );
}


//-------------------------------------
// function : SwingEventServer::OnJavaShutdown
// brief    : notifies all control readers that the JVM is to be killed
//-------------------------------------
void
SwingEventServer::OnJavaShutdown( long vmID )
{
	ControlReaders::iterator it;

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		reader->OnJavaShutdown( vmID );
	}
}


//-------------------------------------
// function : SwingEventServer::OnMenuDeselected
// brief    : notifies all control readers that a menu was deselected
// notes    : "Menu" means menu item with childs only.
//-------------------------------------
void
SwingEventServer::OnMenuDeselected( long vmID, jobject event, AccessibleContext context )
{
	BOOL rc; // result code
	AccessibleContextInfo contextInfo;
	ControlReaders::iterator it;

	// get context information
	rc = GetAccessibleContextInfo( vmID, context, &contextInfo );
	if( rc == FALSE )
	{
		goto cleanup;
	}

	TRACE_PRINT("OnMenuDeselected (%s)\n", Utl::Narrow(contextInfo.name).c_str());

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		if( reader->CanHandleContext(vmID, context, contextInfo) )
		{
			reader->OnMenuDeselected( vmID, event, context, contextInfo );
			break; // only one reader can handle event
		}
	}

cleanup:
	// dispose Java objects
	ReleaseJavaObject( vmID, event );
	ReleaseJavaObject( vmID, context );
}

//-------------------------------------
// function : SwingEventServer::OnMenuSelected
// brief    : notifies all control readers that a menu was selected
// notes    : "Menu" means menu item with childs only.
//-------------------------------------
void
SwingEventServer::OnMenuSelected( long vmID, jobject event, AccessibleContext context )
{
	BOOL rc; // result code
	AccessibleContextInfo contextInfo;
	ControlReaders::iterator it;

	// get context information
	rc = GetAccessibleContextInfo( vmID, context, &contextInfo );
	if( rc == FALSE )
	{
		goto cleanup;
	}

	TRACE_PRINT("OnMenuSelected (%s)\n", Utl::Narrow(contextInfo.name).c_str());

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		if( reader->CanHandleContext(vmID, context, contextInfo) )
		{
			reader->OnMenuSelected( vmID, event, context, contextInfo );
			break; // only one reader can handle event
		}
	}

cleanup:
	// dispose Java objects
	ReleaseJavaObject( vmID, event );
	ReleaseJavaObject( vmID, context );
}

//-------------------------------------
// function : SwingEventServer::OnCaretUpdate
// brief    : notifies all control readers that the caret has updated 
//-------------------------------------
void
SwingEventServer::OnCaretUpdate( long vmID, CaretEvent event, AccessibleContext context )
{
	BOOL rc; // result code
	AccessibleContextInfo contextInfo;
	ControlReaders::iterator it;

	// get context information
	rc = GetAccessibleContextInfo( vmID, context, &contextInfo );
	if( rc == FALSE )
	{
		goto cleanup;
	}

	TRACE_PRINT( "OnCaretUpdate (%s)\n", Utl::Narrow(contextInfo.name).c_str() );

	// ignore events from unfocused component
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		return;
	}

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		if( reader->CanHandleContext(vmID, context, contextInfo) )
		{
			reader->OnCaretUpdate( vmID, event, context, contextInfo );
		}
	}

cleanup:
	// dispose Java objects
	ReleaseJavaObject( vmID, event );
	ReleaseJavaObject( vmID, context );
}


//-------------------------------------
// function : SwingEventServer::OnCaretPropertyChange
// brief    : notifies all control readers that the caret state was changed
//-------------------------------------
void
SwingEventServer::OnCaretPropertyChange( long vmID, CaretEvent event, AccessibleContext context, int oldPosition, int newPosition )
{
	BOOL rc; // result code
	AccessibleContextInfo contextInfo;
	ControlReaders::iterator it;

	// get context information
	rc = GetAccessibleContextInfo( vmID, context, &contextInfo );
	if( rc == FALSE )
	{
		goto cleanup;
	}

	TRACE_PRINT( "OnCaretPropertyChange (%s)\n", Utl::Narrow(contextInfo.name).c_str() );

	// ignore events from unfocused component
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		return;
	}

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		if( reader->CanHandleContext(vmID, context, contextInfo) )
		{
			reader->OnCaretPropertyChange( vmID, event, context, contextInfo, oldPosition, newPosition );
			goto cleanup; // only one reader
		}
	}

cleanup:
	// dispose Java object
	ReleaseJavaObject( vmID, event );
	ReleaseJavaObject( vmID, context );
}


//-------------------------------------
// function : SwingEventServer::OnStatePropertyChange
// brief    : notifies all control readers that state of a component was changed
//-------------------------------------
void
SwingEventServer::OnStatePropertyChange( long vmID, jobject event, AccessibleContext context, wchar_t * oldState, wchar_t * newState )
{
	BOOL rc; // result code
	AccessibleContextInfo contextInfo;
	ControlReaders::iterator it;

	// get context information
	rc = GetAccessibleContextInfo( vmID, context, &contextInfo );
	if( rc == FALSE )
	{
		goto cleanup;
	}

	TRACE_PRINT("OnStatePropertyChange (event:%d for:%s)\n    %s -> %s\n", event, Utl::Narrow(contextInfo.name).c_str(), Utl::Narrow(oldState).c_str(), Utl::Narrow(newState).c_str());

	// ignore event for unfocused window
	if( wcsstr(contextInfo.states_en_US, L"focused") == 0 )
	{
		// when the node expand/collapsed event happenes in JTreeView,
		// JTreeView has no focus. in such case, do not ignore.
		if( wcsstr(contextInfo.role_en_US, ACCESSIBLE_TREE) != 0 )
		{
			return;
		}
	}

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		if( reader->CanHandleContext(vmID, context, contextInfo) )
		{
			reader->OnStatePropertyChange( vmID, event, context, contextInfo, oldState, newState );
		}
	}

cleanup:
	// dispose Java object
	ReleaseJavaObject( vmID, event );
	ReleaseJavaObject( vmID, context );
}

//-------------------------------------
// function : SwingEventServer::OnSelectionPropertyChange
// brief    : notifies all control readers that selection in a component was changed
//-------------------------------------
void
SwingEventServer::OnSelectionPropertyChange( long vmID, jobject event, AccessibleContext context )
{
	BOOL rc; // result code
	AccessibleContextInfo contextInfo;
	ControlReaders::iterator it;

	// get context information
	rc = GetAccessibleContextInfo( vmID, context, &contextInfo );
	if( rc == FALSE )
	{
		goto cleanup;
	}

	TRACE_PRINT("OnSelectionPropertyChange (%s)\n", Utl::Narrow(contextInfo.name).c_str());

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		if( reader->CanHandleContext(vmID, context, contextInfo) )
		{
			reader->OnSelectionPropertyChange( vmID, event, context, contextInfo );
		}
	}

cleanup:
	// dispose Java object
	ReleaseJavaObject( vmID, event );
	ReleaseJavaObject( vmID, context );
}

//-------------------------------------
// function : SwingEventServer::OnActiveDescendentPropertyChange
// brief    : notifies all control readers that
//            an another descendent was activated in a container component.
//-------------------------------------
void
SwingEventServer::OnActiveDescendentPropertyChange( long vmID, jobject event, AccessibleContext context, jobject oldActiveDescendent, jobject newActiveDescendent )
{
	BOOL rc; // result code
	AccessibleContextInfo contextInfo;
	ControlReaders::iterator it;

	// get context information
	rc = GetAccessibleContextInfo( vmID, context, &contextInfo );
	if( rc == FALSE )
	{
		goto cleanup;
	}

	TRACE_PRINT("OnActiveDescendentPropertyChange (%s)\n", Utl::Narrow(contextInfo.name).c_str());

	// broadcast event for each control readers
	for( it=_ControlReaders.begin(); it!=_ControlReaders.end(); it++ )
	{
		ControlReader * reader = (*it);
		if( reader->CanHandleContext(vmID, context, contextInfo) )
		{
			reader->OnActiveDescendentPropertyChange( vmID, event, context, contextInfo, oldActiveDescendent, newActiveDescendent );
		}
	}

cleanup:
	// dispose Java object
	ReleaseJavaObject( vmID, event );
	ReleaseJavaObject( vmID, context );
}


inline static void
TRACE_PRINT( const char * format, ... )
{
#	ifdef TRACE_EVENT
	va_list argumentVector;
	char logString[1024];

	// write message
	va_start( argumentVector, format );
	vsprintf( logString, format, argumentVector );
	fwrite( logString, 1/*sizeof(char)*/, strlen(logString), stderr );
	va_end( argumentVector );
#	else
	format; // do nothing
#	endif
}


/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
