/* file		Utilities.cpp
 * brief	defines project common utilities
 * date		2007-06-15 (SGRY)
 * license	MIT License (see end of this file)
 **********************************************/
#include "Utilities.h"
#include "Sgry.h"
#include <assert.h>

namespace Utl {

Sgry::Ini gStringResource;


//-------------------------------------
//   function	LoadStringResource
//! @brief		load strings from resource file
//-------------------------------------
void
LoadStringResource()
{
	int		rc; // result code
	char	path[ MAX_PATH ];
	char	language[ 256 ];
	char	country[ 256 ];
	char *	bsPos;
	
	// get current locale
#	ifdef _WIN32
	rc = GetLocaleInfoA( LOCALE_USER_DEFAULT, LOCALE_SISO639LANGNAME, language, 256 );
	if( rc == 0 )
		return;
	rc = GetLocaleInfoA( LOCALE_USER_DEFAULT, LOCALE_SISO3166CTRYNAME, country, 256 );
	if( rc == 0 )
		return;
#	else
#	error not supported environment
#	endif

	// make path of the resource file
#	ifdef _WIN32
	::GetModuleFileNameA( NULL, path, MAX_PATH );
	bsPos = strrchr( path, '\\' );
	if( bsPos == NULL )
	{
		return;
	}
	*bsPos = 0;
#	else
#	error not supported environment
#	endif

	sprintf( path, "%s\\lang\\%s-%s.ini", path, language, country );

	gStringResource.load( path );
}

//-------------------------------------
//   function	GetString
//! @brief		get string resource by name
//-------------------------------------
wstring
GetString( const char* resourceName, const char* defaultValue )
{
	wstring str;
	
	// try get string from INI
	str = gStringResource.get( "", resourceName );
	if( str.empty() )
	{
		return Sgry::Ini::widen( defaultValue ); // return default
	}

	return str;
}

//-------------------------------------
//   function	GetLabelFor
//! @brief		get label text for a component
//! @retval		(success) ... formatted string
//! @retval		(failure) ... empty string
//-------------------------------------
wstring
GetLabelFor( long vmID, AccessibleContext context, int indexInParent )
{
	int rc; // result code
	int index;
	AccessibleContext parent;
	AccessibleContext prevComponent;
	AccessibleContextInfo prevComponentInfo;
	
	// get parent
	parent = GetAccessibleParentFromContext( vmID, context );
	if( parent == 0 )
	{
		goto use_window_title;
	}

	// get the previous component which is not filler
	index = indexInParent - 1;
	while( 0 <= index )
	{
		rc = GetChildInfo( vmID, parent, index, prevComponent, prevComponentInfo );
		if( rc != 0 )
		{
			goto use_window_title;
		}
		if( wcscmp(prevComponentInfo.role_en_US, ACCESSIBLE_FILLER) != 0 )
		{
			break;
		}

		index--;
	}

	// ensure it's a label
	if( wcscmp(prevComponentInfo.role_en_US, ACCESSIBLE_LABEL) != 0 )
	{
		goto use_window_title;
	}

	// finally, get it's text
	return wstring( prevComponentInfo.name );

use_window_title:
	//---- previous component is not a label. so use window-title for label ----
	AccessibleContextInfo	info;

	parent = GetAccessibleParentFromContext( vmID, context );
	while( parent != 0 )
	{
		context = parent;

		parent = GetAccessibleParentFromContext( vmID, context );
	}
	GetAccessibleContextInfo( vmID, context, &info );

	return wstring( info.name );
}

int GetChildInfo( long vmID, AccessibleContext & parent, int childIndex, AccessibleContext & out_prev, AccessibleContextInfo & out_prevInfo )
{
	BOOL rc;

	out_prev = GetAccessibleChildFromContext( vmID, parent, childIndex );
	if( out_prev == 0 )
	{
		return -1;
	}

	// get information about the previous component
	rc = GetAccessibleContextInfo( vmID, out_prev, &out_prevInfo );
	if( rc == FALSE )
	{
		return -1;
	}

	return 0;
}

bool EndsWith( const wstring & str, const wchar_t * subStr )
{
	return EndsWith( str.c_str(), subStr );
}

//-------------------------------------
//   function	EndsWith
//! @brief		determine if the string ends with another string
//-------------------------------------
bool EndsWith( const wchar_t * str, const wchar_t * subStr )
{
	size_t strLen, subStrLen;
	const wchar_t * p;
	
	// get length of the two stirng
	strLen = wcslen( str );
	subStrLen = wcslen( subStr );
	if( strLen < subStrLen )
	{
		return false; // if subStr is longer than the str, that cannot contain subStr
	}

	// do match
	p = wcsstr( str + strLen - subStrLen, subStr );
	if( p == NULL )
	{
		return false;
	}

	return true;
}

//-------------------------------------
//   function	Format
//! @brief		create formatted string with ordered parameters
//! @param		format ... format with ordered parameters like "{0}".
//! @returns	(success) ... formatted string
//!				(failure) ... empty string
//! @note		Parameter must be Sgry::wchar*. Cannot handle over 2k bytes string.
//-------------------------------------
wstring
Format( const wchar_t * format, ... )
{
	const int MAX_PARAM_COUNT = 10;
	int			i;
	wchar_t		buf[256];
	va_list		argList;
	wchar_t *	args[MAX_PARAM_COUNT];
	wchar_t		result[2048];
	wchar_t		paramToken[64];
	int			paramCount;
	int			paramNum;
	const wchar_t * paramPositions[MAX_PARAM_COUNT] = {0};
	const wchar_t * paramPos = NULL;
	const wchar_t *begin, *end;

	result[0] = 0;

	// count parameters in format
	paramCount = 0;
	do
	{
		swprintf( paramToken, L"{%d}", paramCount );
		paramPos = wcsstr( format, paramToken );
		if( paramPos != NULL )
		{
			paramPositions[paramCount] = paramPos;
			paramCount++;
		}
	}
	while( paramCount < MAX_PARAM_COUNT && paramPos != NULL );

	// get same number of arguments
	va_start( argList, format );
	for( i=0; i<paramCount; i++ )
	{
		// get next argument
		args[i] = va_arg( argList, wchar_t* );
	}
	va_end( argList );

	// for each param, copy before param and the param
	begin = format;
	for( i=0; i<paramCount; i++ )
	{
		// find next param in format
		end = wcschr( begin, L'{' );
		if( end == NULL )
		{
			break;
		}

		// copy before the param
		assert( end-begin < sizeof(buf) );
		wcsncpy( buf, begin, end-begin );
		buf[ end-begin ] = 0;
		assert( wcslen(result) + wcslen(buf) < sizeof(result) );
		wcscat( result, buf );

		// distinguish the parameter number
		paramNum = *(end+1) - L'0';
		if( paramNum < 0 )
		{
			break;
		}

		// replace param
		assert( wcslen(result) + wcslen(args[paramNum]) < sizeof(result) );
		wcscat( result, args[paramNum] );

		// go next token
		begin = end + 3; // 3 == wcslen("{0}")
	}

	// finally copy after last param
	assert( wcslen(result) + wcslen(begin) < sizeof(result) );
	wcscat( result, begin );

	// return formatted string
	return wstring( result );
}

//-------------------------------------
//   function	Narrow
//! @brief		Convert wstring to string.
//! @retval		(success) ... converted string object
//! @retval		(failure) ... empty string
//-------------------------------------
string
Narrow( const wstring & str )
{
#	ifdef _WIN32
	int			rc; // result code
	char *		charBuf;
	long		neededBufSize;
	string		narrowStr;
	const UINT	codepage = CP_ACP;

	// get buffer size needed
	neededBufSize = WideCharToMultiByte(
			codepage, 0,
			(const WCHAR*)str.c_str(), (int)str.length(),
			NULL, 0,
			NULL, NULL );
	if( neededBufSize == 0 )
	{
		return string(); // invalid string for conversation?
	}

	// allocate buffer
	charBuf = new char[neededBufSize+1];

	// convert code page
	rc = WideCharToMultiByte(
			codepage, 0,
			(LPCWSTR)str.c_str(), (int)str.length(),
			charBuf, neededBufSize,
			NULL, NULL );
	if( rc == FALSE )
	{
		return string(); // failed to convert
	}
	charBuf[neededBufSize] = 0; // null terminate

	// make string
	narrowStr = charBuf;

	// delete buffer and return value
	delete charBuf;
	return narrowStr;
#	endif
}

} // namespace Utl

/**********************************************************
Copyright (c) 2006-2007 YAMAMOTO Suguru

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
**********************************************************/
